@@ -1,20 +1,27 @@
+#
+# This file is licensed under the Affero General Public License (AGPL) version 3.
+#
 # Copyright 2014-2021 The Matrix.org Foundation C.I.C.
+# Copyright (C) 2023 New Vector, Ltd
 #
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+#
+# See the GNU Affero General Public License for more details:
+# <https://www.gnu.org/licenses/agpl-3.0.html>.
+#
+# Originally licensed under the Apache License, Version 2.0:
+# <http://www.apache.org/licenses/LICENSE-2.0>.
+#
+# [This file includes modifications made by New Vector Limited]
 #
-#     http://www.apache.org/licenses/LICENSE-2.0
 #
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 
 import abc
 import logging
-from typing import TYPE_CHECKING, Callable, Dict, Iterable, List, Optional, Tuple
+from typing import TYPE_CHECKING, Callable, Iterable
 
 import attr
 from signedjson.key import (
@@ -23,12 +30,7 @@ from signedjson.key import (
     get_verify_key,
     is_signing_algorithm_supported,
 )
-from signedjson.sign import (
-    SignatureVerifyException,
-    encode_canonical_json,
-    signature_ids,
-    verify_signed_json,
-)
+from signedjson.sign import SignatureVerifyException, signature_ids, verify_signed_json
 from signedjson.types import VerifyKey
 from unpaddedbase64 import decode_base64
 
@@ -80,7 +82,7 @@ class VerifyJsonRequest:
     server_name: str
     get_json_object: Callable[[], JsonDict]
     minimum_valid_until_ts: int
-    key_ids: List[str]
+    key_ids: list[str]
 
     @staticmethod
     def from_json_object(
@@ -139,7 +141,7 @@ class _FetchKeyRequest:
 
     server_name: str
     minimum_valid_until_ts: int
-    key_ids: List[str]
+    key_ids: list[str]
 
 
 class Keyring:
@@ -148,11 +150,13 @@ class Keyring:
     """
 
     def __init__(
-        self, hs: "HomeServer", key_fetchers: "Optional[Iterable[KeyFetcher]]" = None
+        self, hs: "HomeServer", key_fetchers: "Iterable[KeyFetcher] | None" = None
     ):
+        self.server_name = hs.hostname
+
         if key_fetchers is None:
             # Always fetch keys from the database.
-            mutable_key_fetchers: List[KeyFetcher] = [StoreKeyFetcher(hs)]
+            mutable_key_fetchers: list[KeyFetcher] = [StoreKeyFetcher(hs)]
             # Fetch keys from configured trusted key servers, if any exist.
             key_servers = hs.config.key.key_servers
             if key_servers:
@@ -165,9 +169,10 @@ class Keyring:
             self._key_fetchers = key_fetchers
 
         self._fetch_keys_queue: BatchingQueue[
-            _FetchKeyRequest, Dict[str, Dict[str, FetchKeyResult]]
+            _FetchKeyRequest, dict[str, dict[str, FetchKeyResult]]
         ] = BatchingQueue(
-            "keyring_server",
+            name="keyring_server",
+            hs=hs,
             clock=hs.get_clock(),
             # The method called to fetch each key
             process_batch_callback=self._inner_fetch_key_requests,
@@ -177,7 +182,7 @@ class Keyring:
 
         # build a FetchKeyResult for each of our own keys, to shortcircuit the
         # fetcher.
-        self._local_verify_keys: Dict[str, FetchKeyResult] = {}
+        self._local_verify_keys: dict[str, FetchKeyResult] = {}
         for key_id, key in hs.config.key.old_signing_keys.items():
             self._local_verify_keys[key_id] = FetchKeyResult(
                 verify_key=key, valid_until_ts=key.expired
@@ -189,6 +194,14 @@ class Keyring:
             valid_until_ts=2**63,  # fake future timestamp
         )
 
+    def shutdown(self) -> None:
+        """
+        Prepares the KeyRing for garbage collection by shutting down it's queues.
+        """
+        self._fetch_keys_queue.shutdown()
+        for key_fetcher in self._key_fetchers:
+            key_fetcher.shutdown()
+
     async def verify_json_for_server(
         self,
         server_name: str,
@@ -216,8 +229,8 @@ class Keyring:
         return await self.process_request(request)
 
     def verify_json_objects_for_server(
-        self, server_and_json: Iterable[Tuple[str, dict, int]]
-    ) -> List["defer.Deferred[None]"]:
+        self, server_and_json: Iterable[tuple[str, dict, int]]
+    ) -> list["defer.Deferred[None]"]:
         """Bulk verifies signatures of json objects, bulk fetching keys as
         necessary.
 
@@ -273,7 +286,7 @@ class Keyring:
                 Codes.UNAUTHORIZED,
             )
 
-        found_keys: Dict[str, FetchKeyResult] = {}
+        found_keys: dict[str, FetchKeyResult] = {}
 
         # If we are the originating server, short-circuit the key-fetch for any keys
         # we already have
@@ -311,7 +324,7 @@ class Keyring:
             if key_result.valid_until_ts < verify_request.minimum_valid_until_ts:
                 continue
 
-            await self._process_json(key_result.verify_key, verify_request)
+            await self.process_json(key_result.verify_key, verify_request)
             verified = True
 
         if not verified:
@@ -321,7 +334,7 @@ class Keyring:
                 Codes.UNAUTHORIZED,
             )
 
-    async def _process_json(
+    async def process_json(
         self, verify_key: VerifyKey, verify_request: VerifyJsonRequest
     ) -> None:
         """Processes the `VerifyJsonRequest`. Raises if the signature can't be
@@ -355,8 +368,8 @@ class Keyring:
             )
 
     async def _inner_fetch_key_requests(
-        self, requests: List[_FetchKeyRequest]
-    ) -> Dict[str, Dict[str, FetchKeyResult]]:
+        self, requests: list[_FetchKeyRequest]
+    ) -> dict[str, dict[str, FetchKeyResult]]:
         """Processing function for the queue of `_FetchKeyRequest`.
 
         Takes a list of key fetch requests, de-duplicates them and then carries out
@@ -374,7 +387,7 @@ class Keyring:
         # First we need to deduplicate requests for the same key. We do this by
         # taking the *maximum* requested `minimum_valid_until_ts` for each pair
         # of server name/key ID.
-        server_to_key_to_ts: Dict[str, Dict[str, int]] = {}
+        server_to_key_to_ts: dict[str, dict[str, int]] = {}
         for request in requests:
             by_server = server_to_key_to_ts.setdefault(request.server_name, {})
             for key_id in request.key_ids:
@@ -399,7 +412,7 @@ class Keyring:
 
         # We now convert the returned list of results into a map from server
         # name to key ID to FetchKeyResult, to return.
-        to_return: Dict[str, Dict[str, FetchKeyResult]] = {}
+        to_return: dict[str, dict[str, FetchKeyResult]] = {}
         for request, results in zip(deduped_requests, results_per_request):
             to_return_by_server = to_return.setdefault(request.server_name, {})
             for key_id, key_result in results.items():
@@ -411,7 +424,7 @@ class Keyring:
 
     async def _inner_fetch_key_request(
         self, verify_request: _FetchKeyRequest
-    ) -> Dict[str, FetchKeyResult]:
+    ) -> dict[str, FetchKeyResult]:
         """Attempt to fetch the given key by calling each key fetcher one by one.
 
         If a key is found, check whether its `valid_until_ts` attribute satisfies the
@@ -432,7 +445,7 @@ class Keyring:
         """
         logger.debug("Starting fetch for %s", verify_request)
 
-        found_keys: Dict[str, FetchKeyResult] = {}
+        found_keys: dict[str, FetchKeyResult] = {}
         missing_key_ids = set(verify_request.key_ids)
 
         for fetcher in self._key_fetchers:
@@ -471,13 +484,23 @@ class Keyring:
 
 class KeyFetcher(metaclass=abc.ABCMeta):
     def __init__(self, hs: "HomeServer"):
+        self.server_name = hs.hostname
         self._queue = BatchingQueue(
-            self.__class__.__name__, hs.get_clock(), self._fetch_keys
+            name=self.__class__.__name__,
+            hs=hs,
+            clock=hs.get_clock(),
+            process_batch_callback=self._fetch_keys,
         )
 
+    def shutdown(self) -> None:
+        """
+        Prepares the KeyFetcher for garbage collection by shutting down it's queue.
+        """
+        self._queue.shutdown()
+
     async def get_keys(
-        self, server_name: str, key_ids: List[str], minimum_valid_until_ts: int
-    ) -> Dict[str, FetchKeyResult]:
+        self, server_name: str, key_ids: list[str], minimum_valid_until_ts: int
+    ) -> dict[str, FetchKeyResult]:
         results = await self._queue.add_to_queue(
             _FetchKeyRequest(
                 server_name=server_name,
@@ -489,8 +512,8 @@ class KeyFetcher(metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     async def _fetch_keys(
-        self, keys_to_fetch: List[_FetchKeyRequest]
-    ) -> Dict[str, Dict[str, FetchKeyResult]]:
+        self, keys_to_fetch: list[_FetchKeyRequest]
+    ) -> dict[str, dict[str, FetchKeyResult]]:
         pass
 
 
@@ -503,8 +526,8 @@ class StoreKeyFetcher(KeyFetcher):
         self.store = hs.get_datastores().main
 
     async def _fetch_keys(
-        self, keys_to_fetch: List[_FetchKeyRequest]
-    ) -> Dict[str, Dict[str, FetchKeyResult]]:
+        self, keys_to_fetch: list[_FetchKeyRequest]
+    ) -> dict[str, dict[str, FetchKeyResult]]:
         key_ids_to_fetch = (
             (queue_value.server_name, key_id)
             for queue_value in keys_to_fetch
@@ -512,7 +535,7 @@ class StoreKeyFetcher(KeyFetcher):
         )
 
         res = await self.store.get_server_keys_json(key_ids_to_fetch)
-        keys: Dict[str, Dict[str, FetchKeyResult]] = {}
+        keys: dict[str, dict[str, FetchKeyResult]] = {}
         for (server_name, key_id), key in res.items():
             keys.setdefault(server_name, {})[key_id] = key
         return keys
@@ -526,7 +549,7 @@ class BaseV2KeyFetcher(KeyFetcher):
 
     async def process_v2_response(
         self, from_server: str, response_json: JsonDict, time_added_ms: int
-    ) -> Dict[str, FetchKeyResult]:
+    ) -> dict[str, FetchKeyResult]:
         """Parse a 'Server Keys' structure from the result of a /key request
 
         This is used to parse either the entirety of the response from
@@ -587,7 +610,7 @@ class BaseV2KeyFetcher(KeyFetcher):
                 % (server_name,)
             )
 
-        for key_id, key_data in response_json["old_verify_keys"].items():
+        for key_id, key_data in response_json.get("old_verify_keys", {}).items():
             if is_signing_algorithm_supported(key_id):
                 key_base64 = key_data["key"]
                 key_bytes = decode_base64(key_base64)
@@ -596,24 +619,12 @@ class BaseV2KeyFetcher(KeyFetcher):
                     verify_key=verify_key, valid_until_ts=key_data["expired_ts"]
                 )
 
-        key_json_bytes = encode_canonical_json(response_json)
-
-        await make_deferred_yieldable(
-            defer.gatherResults(
-                [
-                    run_in_background(
-                        self.store.store_server_keys_json,
-                        server_name=server_name,
-                        key_id=key_id,
-                        from_server=from_server,
-                        ts_now_ms=time_added_ms,
-                        ts_expires_ms=ts_valid_until_ms,
-                        key_json_bytes=key_json_bytes,
-                    )
-                    for key_id in verify_keys
-                ],
-                consumeErrors=True,
-            ).addErrback(unwrapFirstError)
+        await self.store.store_server_keys_response(
+            server_name=server_name,
+            from_server=from_server,
+            ts_added_ms=time_added_ms,
+            verify_keys=verify_keys,
+            response_json=response_json,
         )
 
         return verify_keys
@@ -629,11 +640,11 @@ class PerspectivesKeyFetcher(BaseV2KeyFetcher):
         self.key_servers = hs.config.key.key_servers
 
     async def _fetch_keys(
-        self, keys_to_fetch: List[_FetchKeyRequest]
-    ) -> Dict[str, Dict[str, FetchKeyResult]]:
+        self, keys_to_fetch: list[_FetchKeyRequest]
+    ) -> dict[str, dict[str, FetchKeyResult]]:
         """see KeyFetcher._fetch_keys"""
 
-        async def get_key(key_server: TrustedKeyServer) -> Dict:
+        async def get_key(key_server: TrustedKeyServer) -> dict:
             try:
                 return await self.get_server_verify_key_v2_indirect(
                     keys_to_fetch, key_server
@@ -659,7 +670,7 @@ class PerspectivesKeyFetcher(BaseV2KeyFetcher):
             ).addErrback(unwrapFirstError)
         )
 
-        union_of_keys: Dict[str, Dict[str, FetchKeyResult]] = {}
+        union_of_keys: dict[str, dict[str, FetchKeyResult]] = {}
         for result in results:
             for server_name, keys in result.items():
                 union_of_keys.setdefault(server_name, {}).update(keys)
@@ -667,8 +678,8 @@ class PerspectivesKeyFetcher(BaseV2KeyFetcher):
         return union_of_keys
 
     async def get_server_verify_key_v2_indirect(
-        self, keys_to_fetch: List[_FetchKeyRequest], key_server: TrustedKeyServer
-    ) -> Dict[str, Dict[str, FetchKeyResult]]:
+        self, keys_to_fetch: list[_FetchKeyRequest], key_server: TrustedKeyServer
+    ) -> dict[str, dict[str, FetchKeyResult]]:
         """
         Args:
             keys_to_fetch:
@@ -720,8 +731,8 @@ class PerspectivesKeyFetcher(BaseV2KeyFetcher):
             "Response from notary server %s: %s", perspective_name, query_response
         )
 
-        keys: Dict[str, Dict[str, FetchKeyResult]] = {}
-        added_keys: Dict[Tuple[str, str], FetchKeyResult] = {}
+        keys: dict[str, dict[str, FetchKeyResult]] = {}
+        added_keys: dict[tuple[str, str], FetchKeyResult] = {}
 
         time_now_ms = self.clock.time_msec()
 
@@ -775,10 +786,6 @@ class PerspectivesKeyFetcher(BaseV2KeyFetcher):
 
             keys.setdefault(server_name, {}).update(processed_response)
 
-        await self.store.store_server_signature_keys(
-            perspective_name, time_now_ms, added_keys
-        )
-
         return keys
 
     def _validate_perspectives_response(
@@ -829,8 +836,8 @@ class ServerKeyFetcher(BaseV2KeyFetcher):
         self.client = hs.get_federation_http_client()
 
     async def get_keys(
-        self, server_name: str, key_ids: List[str], minimum_valid_until_ts: int
-    ) -> Dict[str, FetchKeyResult]:
+        self, server_name: str, key_ids: list[str], minimum_valid_until_ts: int
+    ) -> dict[str, FetchKeyResult]:
         results = await self._queue.add_to_queue(
             _FetchKeyRequest(
                 server_name=server_name,
@@ -842,8 +849,8 @@ class ServerKeyFetcher(BaseV2KeyFetcher):
         return results.get(server_name, {})
 
     async def _fetch_keys(
-        self, keys_to_fetch: List[_FetchKeyRequest]
-    ) -> Dict[str, Dict[str, FetchKeyResult]]:
+        self, keys_to_fetch: list[_FetchKeyRequest]
+    ) -> dict[str, dict[str, FetchKeyResult]]:
         """
         Args:
             keys_to_fetch:
@@ -853,11 +860,12 @@ class ServerKeyFetcher(BaseV2KeyFetcher):
             Map from server_name -> key_id -> FetchKeyResult
         """
 
+        # We only need to do one request per server.
+        servers_to_fetch = {k.server_name for k in keys_to_fetch}
+
         results = {}
 
-        async def get_keys(key_to_fetch_item: _FetchKeyRequest) -> None:
-            server_name = key_to_fetch_item.server_name
-
+        async def get_keys(server_name: str) -> None:
             try:
                 keys = await self.get_server_verify_keys_v2_direct(server_name)
                 results[server_name] = keys
@@ -866,12 +874,12 @@ class ServerKeyFetcher(BaseV2KeyFetcher):
             except Exception:
                 logger.exception("Error getting keys from %s", server_name)
 
-        await yieldable_gather_results(get_keys, keys_to_fetch)
+        await yieldable_gather_results(get_keys, servers_to_fetch)
         return results
 
     async def get_server_verify_keys_v2_direct(
         self, server_name: str
-    ) -> Dict[str, FetchKeyResult]:
+    ) -> dict[str, FetchKeyResult]:
         """
 
         Args:
