@@ -22,18 +22,11 @@ import logging
 from collections import Counter
 from typing import (
     TYPE_CHECKING,
-    Dict,
-    List,
+    Literal,
     Mapping,
-    Optional,
     Sequence,
-    Tuple,
-    Type,
-    Union,
 )
 
-from typing_extensions import Literal
-
 from synapse.api.constants import Direction, EduTypes
 from synapse.api.errors import Codes, SynapseError
 from synapse.api.room_versions import RoomVersions
@@ -44,10 +37,15 @@ from synapse.federation.transport.server._base import (
 )
 from synapse.http.servlet import (
     parse_boolean_from_args,
+    parse_integer,
     parse_integer_from_args,
+    parse_string,
     parse_string_from_args,
     parse_strings_from_args,
 )
+from synapse.http.site import SynapseRequest
+from synapse.media._base import DEFAULT_MAX_TIMEOUT_MS, MAXIMUM_ALLOWED_MAX_TIMEOUT_MS
+from synapse.media.thumbnailer import ThumbnailProvider
 from synapse.types import JsonDict
 from synapse.util import SYNAPSE_VERSION
 from synapse.util.ratelimitutils import FederationRateLimiter
@@ -89,9 +87,9 @@ class FederationSendServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: JsonDict,
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         transaction_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         """Called on PUT /send/<transaction_id>/
 
         Args:
@@ -154,9 +152,9 @@ class FederationEventServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: Literal[None],
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         event_id: str,
-    ) -> Tuple[int, Union[JsonDict, str]]:
+    ) -> tuple[int, JsonDict | str]:
         return await self.handler.on_pdu_request(origin, event_id)
 
 
@@ -169,9 +167,9 @@ class FederationStateV1Servlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: Literal[None],
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         return await self.handler.on_room_state_request(
             origin,
             room_id,
@@ -187,9 +185,9 @@ class FederationStateIdsServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: Literal[None],
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         return await self.handler.on_state_ids_request(
             origin,
             room_id,
@@ -205,9 +203,9 @@ class FederationBackfillServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: Literal[None],
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         versions = [x.decode("ascii") for x in query[b"v"]]
         limit = parse_integer_from_args(query, "limit", None)
 
@@ -244,9 +242,9 @@ class FederationTimestampLookupServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: Literal[None],
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         timestamp = parse_integer_from_args(query, "ts", required=True)
         direction_str = parse_string_from_args(
             query, "dir", allowed_values=["f", "b"], required=True
@@ -267,9 +265,9 @@ class FederationQueryServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: Literal[None],
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         query_type: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         args = {k.decode("utf8"): v[0].decode("utf-8") for k, v in query.items()}
         args["origin"] = origin
         return await self.handler.on_query_request(query_type, args)
@@ -283,10 +281,10 @@ class FederationMakeJoinServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: Literal[None],
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
         user_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         """
         Args:
             origin: The authenticated server_name of the calling server
@@ -319,10 +317,10 @@ class FederationMakeLeaveServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: Literal[None],
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
         user_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         result = await self.handler.on_make_leave_request(origin, room_id, user_id)
         return 200, result
 
@@ -335,10 +333,10 @@ class FederationV1SendLeaveServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: JsonDict,
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
         event_id: str,
-    ) -> Tuple[int, Tuple[int, JsonDict]]:
+    ) -> tuple[int, tuple[int, JsonDict]]:
         result = await self.handler.on_send_leave_request(origin, content, room_id)
         return 200, (200, result)
 
@@ -353,10 +351,10 @@ class FederationV2SendLeaveServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: JsonDict,
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
         event_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         result = await self.handler.on_send_leave_request(origin, content, room_id)
         return 200, result
 
@@ -369,10 +367,10 @@ class FederationMakeKnockServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: Literal[None],
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
         user_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         # Retrieve the room versions the remote homeserver claims to support
         supported_versions = parse_strings_from_args(
             query, "ver", required=True, encoding="utf-8"
@@ -392,10 +390,10 @@ class FederationV1SendKnockServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: JsonDict,
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
         event_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         result = await self.handler.on_send_knock_request(origin, content, room_id)
         return 200, result
 
@@ -408,10 +406,10 @@ class FederationEventAuthServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: Literal[None],
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
         event_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         return await self.handler.on_event_auth(origin, room_id, event_id)
 
 
@@ -423,10 +421,10 @@ class FederationV1SendJoinServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: JsonDict,
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
         event_id: str,
-    ) -> Tuple[int, Tuple[int, JsonDict]]:
+    ) -> tuple[int, tuple[int, JsonDict]]:
         # TODO(paul): assert that event_id parsed from path actually
         #   match those given in content
         result = await self.handler.on_send_join_request(origin, content, room_id)
@@ -443,10 +441,10 @@ class FederationV2SendJoinServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: JsonDict,
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
         event_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         # TODO(paul): assert that event_id parsed from path actually
         #   match those given in content
 
@@ -466,10 +464,10 @@ class FederationV1InviteServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: JsonDict,
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
         event_id: str,
-    ) -> Tuple[int, Tuple[int, JsonDict]]:
+    ) -> tuple[int, tuple[int, JsonDict]]:
         # We don't get a room version, so we have to assume its EITHER v1 or
         # v2. This is "fine" as the only difference between V1 and V2 is the
         # state resolution algorithm, and we don't use that for processing
@@ -493,10 +491,10 @@ class FederationV2InviteServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: JsonDict,
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
         event_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         # TODO(paul): assert that room_id/event_id parsed from path actually
         #   match those given in content
 
@@ -504,6 +502,9 @@ class FederationV2InviteServlet(BaseFederationServerServlet):
         event = content["event"]
         invite_room_state = content.get("invite_room_state", [])
 
+        if not isinstance(invite_room_state, list):
+            invite_room_state = []
+
         # Synapse expects invite_room_state to be in unsigned, as it is in v1
         # API
 
@@ -528,9 +529,9 @@ class FederationThirdPartyInviteExchangeServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: JsonDict,
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         await self.handler.on_exchange_third_party_invite_request(content)
         return 200, {}
 
@@ -540,8 +541,8 @@ class FederationClientKeysQueryServlet(BaseFederationServerServlet):
     CATEGORY = "Federation requests"
 
     async def on_POST(
-        self, origin: str, content: JsonDict, query: Dict[bytes, List[bytes]]
-    ) -> Tuple[int, JsonDict]:
+        self, origin: str, content: JsonDict, query: dict[bytes, list[bytes]]
+    ) -> tuple[int, JsonDict]:
         return await self.handler.on_query_client_keys(origin, content)
 
 
@@ -553,9 +554,9 @@ class FederationUserDevicesQueryServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: Literal[None],
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         user_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         return await self.handler.on_query_user_devices(origin, user_id)
 
 
@@ -564,10 +565,10 @@ class FederationClientKeysClaimServlet(BaseFederationServerServlet):
     CATEGORY = "Federation requests"
 
     async def on_POST(
-        self, origin: str, content: JsonDict, query: Dict[bytes, List[bytes]]
-    ) -> Tuple[int, JsonDict]:
+        self, origin: str, content: JsonDict, query: dict[bytes, list[bytes]]
+    ) -> tuple[int, JsonDict]:
         # Generate a count for each algorithm, which is hard-coded to 1.
-        key_query: List[Tuple[str, str, str, int]] = []
+        key_query: list[tuple[str, str, str, int]] = []
         for user_id, device_keys in content.get("one_time_keys", {}).items():
             for device_id, algorithm in device_keys.items():
                 key_query.append((user_id, device_id, algorithm, 1))
@@ -590,10 +591,10 @@ class FederationUnstableClientKeysClaimServlet(BaseFederationServerServlet):
     CATEGORY = "Federation requests"
 
     async def on_POST(
-        self, origin: str, content: JsonDict, query: Dict[bytes, List[bytes]]
-    ) -> Tuple[int, JsonDict]:
+        self, origin: str, content: JsonDict, query: dict[bytes, list[bytes]]
+    ) -> tuple[int, JsonDict]:
         # Generate a count for each algorithm.
-        key_query: List[Tuple[str, str, str, int]] = []
+        key_query: list[tuple[str, str, str, int]] = []
         for user_id, device_keys in content.get("one_time_keys", {}).items():
             for device_id, algorithms in device_keys.items():
                 counts = Counter(algorithms)
@@ -614,9 +615,9 @@ class FederationGetMissingEventsServlet(BaseFederationServerServlet):
         self,
         origin: str,
         content: JsonDict,
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         limit = int(content.get("limit", 10))
         earliest_events = content.get("earliest_events", [])
         latest_events = content.get("latest_events", [])
@@ -639,8 +640,8 @@ class On3pidBindServlet(BaseFederationServerServlet):
     REQUIRE_AUTH = False
 
     async def on_POST(
-        self, origin: Optional[str], content: JsonDict, query: Dict[bytes, List[bytes]]
-    ) -> Tuple[int, JsonDict]:
+        self, origin: str | None, content: JsonDict, query: dict[bytes, list[bytes]]
+    ) -> tuple[int, JsonDict]:
         if "invites" in content:
             last_exception = None
             for invite in content["invites"]:
@@ -673,10 +674,10 @@ class FederationVersionServlet(BaseFederationServlet):
 
     async def on_GET(
         self,
-        origin: Optional[str],
+        origin: str | None,
         content: Literal[None],
-        query: Dict[bytes, List[bytes]],
-    ) -> Tuple[int, JsonDict]:
+        query: dict[bytes, list[bytes]],
+    ) -> tuple[int, JsonDict]:
         return (
             200,
             {
@@ -708,7 +709,7 @@ class FederationRoomHierarchyServlet(BaseFederationServlet):
         content: Literal[None],
         query: Mapping[bytes, Sequence[bytes]],
         room_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         suggested_only = parse_boolean_from_args(query, "suggested_only", default=False)
         return 200, await self.handler.get_federation_hierarchy(
             origin, room_id, suggested_only
@@ -739,9 +740,9 @@ class RoomComplexityServlet(BaseFederationServlet):
         self,
         origin: str,
         content: Literal[None],
-        query: Dict[bytes, List[bytes]],
+        query: dict[bytes, list[bytes]],
         room_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         is_public = await self._store.is_room_world_readable_or_publicly_joinable(
             room_id
         )
@@ -773,7 +774,7 @@ class FederationAccountStatusServlet(BaseFederationServerServlet):
         content: JsonDict,
         query: Mapping[bytes, Sequence[bytes]],
         room_id: str,
-    ) -> Tuple[int, JsonDict]:
+    ) -> tuple[int, JsonDict]:
         if "user_ids" not in content:
             raise SynapseError(
                 400, "Required parameter 'user_ids' is missing", Codes.MISSING_PARAM
@@ -787,7 +788,95 @@ class FederationAccountStatusServlet(BaseFederationServerServlet):
         return 200, {"account_statuses": statuses, "failures": failures}
 
 
-FEDERATION_SERVLET_CLASSES: Tuple[Type[BaseFederationServlet], ...] = (
+class FederationMediaDownloadServlet(BaseFederationServerServlet):
+    """
+    Implementation of new federation media `/download` endpoint outlined in MSC3916. Returns
+    a multipart/mixed response consisting of a JSON object and the requested media
+    item. This endpoint only returns local media.
+    """
+
+    PATH = "/media/download/(?P<media_id>[^/]*)"
+    RATELIMIT = True
+
+    def __init__(
+        self,
+        hs: "HomeServer",
+        ratelimiter: FederationRateLimiter,
+        authenticator: Authenticator,
+        server_name: str,
+    ):
+        super().__init__(hs, authenticator, ratelimiter, server_name)
+        self.media_repo = self.hs.get_media_repository()
+
+    async def on_GET(
+        self,
+        origin: str | None,
+        content: Literal[None],
+        request: SynapseRequest,
+        media_id: str,
+    ) -> None:
+        max_timeout_ms = parse_integer(
+            request, "timeout_ms", default=DEFAULT_MAX_TIMEOUT_MS
+        )
+        max_timeout_ms = min(max_timeout_ms, MAXIMUM_ALLOWED_MAX_TIMEOUT_MS)
+        await self.media_repo.get_local_media(
+            request, media_id, None, max_timeout_ms, federation=True
+        )
+
+
+class FederationMediaThumbnailServlet(BaseFederationServerServlet):
+    """
+    Implementation of new federation media `/thumbnail` endpoint outlined in MSC3916. Returns
+    a multipart/mixed response consisting of a JSON object and the requested media
+    item. This endpoint only returns local media.
+    """
+
+    PATH = "/media/thumbnail/(?P<media_id>[^/]*)"
+    RATELIMIT = True
+
+    def __init__(
+        self,
+        hs: "HomeServer",
+        ratelimiter: FederationRateLimiter,
+        authenticator: Authenticator,
+        server_name: str,
+    ):
+        super().__init__(hs, authenticator, ratelimiter, server_name)
+        self.media_repo = self.hs.get_media_repository()
+        self.dynamic_thumbnails = hs.config.media.dynamic_thumbnails
+        self.thumbnail_provider = ThumbnailProvider(
+            hs, self.media_repo, self.media_repo.media_storage
+        )
+
+    async def on_GET(
+        self,
+        origin: str | None,
+        content: Literal[None],
+        request: SynapseRequest,
+        media_id: str,
+    ) -> None:
+        width = parse_integer(request, "width", required=True)
+        height = parse_integer(request, "height", required=True)
+        method = parse_string(request, "method", "scale")
+        # TODO Parse the Accept header to get an prioritised list of thumbnail types.
+        m_type = "image/png"
+        max_timeout_ms = parse_integer(
+            request, "timeout_ms", default=DEFAULT_MAX_TIMEOUT_MS
+        )
+        max_timeout_ms = min(max_timeout_ms, MAXIMUM_ALLOWED_MAX_TIMEOUT_MS)
+
+        if self.dynamic_thumbnails:
+            await self.thumbnail_provider.select_or_generate_local_thumbnail(
+                request, media_id, width, height, method, m_type, max_timeout_ms, True
+            )
+        else:
+            await self.thumbnail_provider.respond_local_thumbnail(
+                request, media_id, width, height, method, m_type, max_timeout_ms, True
+            )
+        self.media_repo.mark_recently_accessed(None, media_id)
+
+
+FEDERATION_SERVLET_CLASSES: tuple[type[BaseFederationServlet], ...] = (
     FederationSendServlet,
     FederationEventServlet,
     FederationStateV1Servlet,
