@@ -21,7 +21,7 @@
 import email.mime.multipart
 import email.utils
 import logging
-from typing import TYPE_CHECKING, List, Optional, Tuple
+from typing import TYPE_CHECKING
 
 from synapse.api.errors import AuthError, StoreError, SynapseError
 from synapse.metrics.background_process_metrics import wrap_as_background_process
@@ -37,7 +37,8 @@ logger = logging.getLogger(__name__)
 
 class AccountValidityHandler:
     def __init__(self, hs: "HomeServer"):
-        self.hs = hs
+        self.hs = hs  # nb must be called this for @wrap_as_background_process
+        self.server_name = hs.hostname
         self.config = hs.config
         self.store = hs.get_datastores().main
         self.send_email_handler = hs.get_send_email_handler()
@@ -107,8 +108,8 @@ class AccountValidityHandler:
     async def on_user_login(
         self,
         user_id: str,
-        auth_provider_type: Optional[str],
-        auth_provider_id: Optional[str],
+        auth_provider_type: str | None,
+        auth_provider_id: str | None,
     ) -> None:
         """Tell third-party modules about a user logins.
 
@@ -221,7 +222,7 @@ class AccountValidityHandler:
 
         await self.store.set_renewal_mail_status(user_id=user_id, email_sent=True)
 
-    async def _get_email_addresses_for_user(self, user_id: str) -> List[str]:
+    async def _get_email_addresses_for_user(self, user_id: str) -> list[str]:
         """Retrieve the list of email addresses attached to a user's account.
 
         Args:
@@ -262,7 +263,7 @@ class AccountValidityHandler:
                 attempts += 1
         raise StoreError(500, "Couldn't generate a unique string as refresh string.")
 
-    async def renew_account(self, renewal_token: str) -> Tuple[bool, bool, int]:
+    async def renew_account(self, renewal_token: str) -> tuple[bool, bool, int]:
         """Renews the account attached to a given renewal token by pushing back the
         expiration date by the current validity period in the server's configuration.
 
@@ -325,9 +326,9 @@ class AccountValidityHandler:
     async def renew_account_for_user(
         self,
         user_id: str,
-        expiration_ts: Optional[int] = None,
+        expiration_ts: int | None = None,
         email_sent: bool = False,
-        renewal_token: Optional[str] = None,
+        renewal_token: str | None = None,
     ) -> int:
         """Renews the account attached to a given user by pushing back the
         expiration date by the current validity period in the server's
