@@ -1,16 +1,23 @@
+#
+# This file is licensed under the Affero General Public License (AGPL) version 3.
+#
 # Copyright 2020 The Matrix.org Foundation C.I.C.
+# Copyright (C) 2023 New Vector, Ltd
 #
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+#
+# See the GNU Affero General Public License for more details:
+# <https://www.gnu.org/licenses/agpl-3.0.html>.
+#
+# Originally licensed under the Apache License, Version 2.0:
+# <http://www.apache.org/licenses/LICENSE-2.0>.
+#
+# [This file includes modifications made by New Vector Limited]
 #
-#     http://www.apache.org/licenses/LICENSE-2.0
 #
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 
 import logging
 import sys
@@ -18,7 +25,7 @@ import traceback
 from collections import deque
 from ipaddress import IPv4Address, IPv6Address, ip_address
 from math import floor
-from typing import Callable, Deque, Optional
+from typing import Callable
 
 import attr
 from zope.interface import implementer
@@ -32,7 +39,7 @@ from twisted.internet.endpoints import (
 )
 from twisted.internet.interfaces import (
     IPushProducer,
-    IReactorTCP,
+    IReactorTime,
     IStreamClientEndpoint,
 )
 from twisted.internet.protocol import Factory, Protocol
@@ -59,7 +66,7 @@ class LogProducer:
     # (connected and registerProducer) which are part of the implementation.
     transport: Connection
     _format: Callable[[logging.LogRecord], str]
-    _buffer: Deque[logging.LogRecord]
+    _buffer: deque[logging.LogRecord]
     _paused: bool = attr.ib(default=False, init=False)
 
     def pauseProducing(self) -> None:
@@ -106,16 +113,16 @@ class RemoteHandler(logging.Handler):
         port: int,
         maximum_buffer: int = 1000,
         level: int = logging.NOTSET,
-        _reactor: Optional[IReactorTCP] = None,
+        _reactor: IReactorTime | None = None,
     ):
         super().__init__(level=level)
         self.host = host
         self.port = port
         self.maximum_buffer = maximum_buffer
 
-        self._buffer: Deque[logging.LogRecord] = deque()
-        self._connection_waiter: Optional[Deferred] = None
-        self._producer: Optional[LogProducer] = None
+        self._buffer: deque[logging.LogRecord] = deque()
+        self._connection_waiter: Deferred | None = None
+        self._producer: LogProducer | None = None
 
         # Connect without DNS lookups if it's a direct IP.
         if _reactor is None:
