@@ -26,15 +26,10 @@ from typing import (
     Awaitable,
     Callable,
     Collection,
-    Dict,
     Generator,
     Iterable,
-    List,
     Mapping,
-    Optional,
-    Tuple,
     TypeVar,
-    Union,
 )
 
 import attr
@@ -230,11 +225,11 @@ class UserIpAndAgent:
 
 def run_as_background_process(
     desc: "LiteralString",
-    func: Callable[..., Awaitable[Optional[T]]],
+    func: Callable[..., Awaitable[T | None]],
     *args: Any,
     bg_start_span: bool = True,
     **kwargs: Any,
-) -> "defer.Deferred[Optional[T]]":
+) -> "defer.Deferred[T | None]":
     """
     XXX: Deprecated: use `ModuleApi.run_as_background_process` instead.
 
@@ -298,8 +293,8 @@ def run_as_background_process(
 def cached(
     *,
     max_entries: int = 1000,
-    num_args: Optional[int] = None,
-    uncached_args: Optional[Collection[str]] = None,
+    num_args: int | None = None,
+    uncached_args: Collection[str] | None = None,
 ) -> Callable[[F], CachedFunction[F]]:
     """Returns a decorator that applies a memoizing cache around the function. This
     decorator behaves similarly to functools.lru_cache.
@@ -341,7 +336,7 @@ class ModuleApi:
 
         # TODO: Fix this type hint once the types for the data stores have been ironed
         #       out.
-        self._store: Union[DataStore, "GenericWorkerStore"] = hs.get_datastores().main
+        self._store: DataStore | "GenericWorkerStore" = hs.get_datastores().main
         self._storage_controllers = hs.get_storage_controllers()
         self._auth = hs.get_auth()
         self._auth_handler = auth_handler
@@ -390,26 +385,20 @@ class ModuleApi:
     def register_spam_checker_callbacks(
         self,
         *,
-        check_event_for_spam: Optional[CHECK_EVENT_FOR_SPAM_CALLBACK] = None,
-        should_drop_federated_event: Optional[
-            SHOULD_DROP_FEDERATED_EVENT_CALLBACK
-        ] = None,
-        user_may_join_room: Optional[USER_MAY_JOIN_ROOM_CALLBACK] = None,
-        user_may_invite: Optional[USER_MAY_INVITE_CALLBACK] = None,
-        federated_user_may_invite: Optional[FEDERATED_USER_MAY_INVITE_CALLBACK] = None,
-        user_may_send_3pid_invite: Optional[USER_MAY_SEND_3PID_INVITE_CALLBACK] = None,
-        user_may_create_room: Optional[USER_MAY_CREATE_ROOM_CALLBACK] = None,
-        user_may_create_room_alias: Optional[
-            USER_MAY_CREATE_ROOM_ALIAS_CALLBACK
-        ] = None,
-        user_may_publish_room: Optional[USER_MAY_PUBLISH_ROOM_CALLBACK] = None,
-        user_may_send_state_event: Optional[USER_MAY_SEND_STATE_EVENT_CALLBACK] = None,
-        check_username_for_spam: Optional[CHECK_USERNAME_FOR_SPAM_CALLBACK] = None,
-        check_registration_for_spam: Optional[
-            CHECK_REGISTRATION_FOR_SPAM_CALLBACK
-        ] = None,
-        check_media_file_for_spam: Optional[CHECK_MEDIA_FILE_FOR_SPAM_CALLBACK] = None,
-        check_login_for_spam: Optional[CHECK_LOGIN_FOR_SPAM_CALLBACK] = None,
+        check_event_for_spam: CHECK_EVENT_FOR_SPAM_CALLBACK | None = None,
+        should_drop_federated_event: SHOULD_DROP_FEDERATED_EVENT_CALLBACK | None = None,
+        user_may_join_room: USER_MAY_JOIN_ROOM_CALLBACK | None = None,
+        user_may_invite: USER_MAY_INVITE_CALLBACK | None = None,
+        federated_user_may_invite: FEDERATED_USER_MAY_INVITE_CALLBACK | None = None,
+        user_may_send_3pid_invite: USER_MAY_SEND_3PID_INVITE_CALLBACK | None = None,
+        user_may_create_room: USER_MAY_CREATE_ROOM_CALLBACK | None = None,
+        user_may_create_room_alias: USER_MAY_CREATE_ROOM_ALIAS_CALLBACK | None = None,
+        user_may_publish_room: USER_MAY_PUBLISH_ROOM_CALLBACK | None = None,
+        user_may_send_state_event: USER_MAY_SEND_STATE_EVENT_CALLBACK | None = None,
+        check_username_for_spam: CHECK_USERNAME_FOR_SPAM_CALLBACK | None = None,
+        check_registration_for_spam: CHECK_REGISTRATION_FOR_SPAM_CALLBACK | None = None,
+        check_media_file_for_spam: CHECK_MEDIA_FILE_FOR_SPAM_CALLBACK | None = None,
+        check_login_for_spam: CHECK_LOGIN_FOR_SPAM_CALLBACK | None = None,
     ) -> None:
         """Registers callbacks for spam checking capabilities.
 
@@ -435,12 +424,12 @@ class ModuleApi:
     def register_account_validity_callbacks(
         self,
         *,
-        is_user_expired: Optional[IS_USER_EXPIRED_CALLBACK] = None,
-        on_user_registration: Optional[ON_USER_REGISTRATION_CALLBACK] = None,
-        on_user_login: Optional[ON_USER_LOGIN_CALLBACK] = None,
-        on_legacy_send_mail: Optional[ON_LEGACY_SEND_MAIL_CALLBACK] = None,
-        on_legacy_renew: Optional[ON_LEGACY_RENEW_CALLBACK] = None,
-        on_legacy_admin_request: Optional[ON_LEGACY_ADMIN_REQUEST] = None,
+        is_user_expired: IS_USER_EXPIRED_CALLBACK | None = None,
+        on_user_registration: ON_USER_REGISTRATION_CALLBACK | None = None,
+        on_user_login: ON_USER_LOGIN_CALLBACK | None = None,
+        on_legacy_send_mail: ON_LEGACY_SEND_MAIL_CALLBACK | None = None,
+        on_legacy_renew: ON_LEGACY_RENEW_CALLBACK | None = None,
+        on_legacy_admin_request: ON_LEGACY_ADMIN_REQUEST | None = None,
     ) -> None:
         """Registers callbacks for account validity capabilities.
 
@@ -458,9 +447,8 @@ class ModuleApi:
     def register_ratelimit_callbacks(
         self,
         *,
-        get_ratelimit_override_for_user: Optional[
-            GET_RATELIMIT_OVERRIDE_FOR_USER_CALLBACK
-        ] = None,
+        get_ratelimit_override_for_user: GET_RATELIMIT_OVERRIDE_FOR_USER_CALLBACK
+        | None = None,
     ) -> None:
         """Registers callbacks for ratelimit capabilities.
         Added in Synapse v1.132.0.
@@ -472,16 +460,13 @@ class ModuleApi:
     def register_media_repository_callbacks(
         self,
         *,
-        get_media_config_for_user: Optional[GET_MEDIA_CONFIG_FOR_USER_CALLBACK] = None,
-        is_user_allowed_to_upload_media_of_size: Optional[
-            IS_USER_ALLOWED_TO_UPLOAD_MEDIA_OF_SIZE_CALLBACK
-        ] = None,
-        get_media_upload_limits_for_user: Optional[
-            GET_MEDIA_UPLOAD_LIMITS_FOR_USER_CALLBACK
-        ] = None,
-        on_media_upload_limit_exceeded: Optional[
-            ON_MEDIA_UPLOAD_LIMIT_EXCEEDED_CALLBACK
-        ] = None,
+        get_media_config_for_user: GET_MEDIA_CONFIG_FOR_USER_CALLBACK | None = None,
+        is_user_allowed_to_upload_media_of_size: IS_USER_ALLOWED_TO_UPLOAD_MEDIA_OF_SIZE_CALLBACK
+        | None = None,
+        get_media_upload_limits_for_user: GET_MEDIA_UPLOAD_LIMITS_FOR_USER_CALLBACK
+        | None = None,
+        on_media_upload_limit_exceeded: ON_MEDIA_UPLOAD_LIMIT_EXCEEDED_CALLBACK
+        | None = None,
     ) -> None:
         """Registers callbacks for media repository capabilities.
         Added in Synapse v1.132.0.
@@ -496,28 +481,23 @@ class ModuleApi:
     def register_third_party_rules_callbacks(
         self,
         *,
-        check_event_allowed: Optional[CHECK_EVENT_ALLOWED_CALLBACK] = None,
-        on_create_room: Optional[ON_CREATE_ROOM_CALLBACK] = None,
-        check_threepid_can_be_invited: Optional[
-            CHECK_THREEPID_CAN_BE_INVITED_CALLBACK
-        ] = None,
-        check_visibility_can_be_modified: Optional[
-            CHECK_VISIBILITY_CAN_BE_MODIFIED_CALLBACK
-        ] = None,
-        on_new_event: Optional[ON_NEW_EVENT_CALLBACK] = None,
-        check_can_shutdown_room: Optional[CHECK_CAN_SHUTDOWN_ROOM_CALLBACK] = None,
-        check_can_deactivate_user: Optional[CHECK_CAN_DEACTIVATE_USER_CALLBACK] = None,
-        on_profile_update: Optional[ON_PROFILE_UPDATE_CALLBACK] = None,
-        on_user_deactivation_status_changed: Optional[
-            ON_USER_DEACTIVATION_STATUS_CHANGED_CALLBACK
-        ] = None,
-        on_threepid_bind: Optional[ON_THREEPID_BIND_CALLBACK] = None,
-        on_add_user_third_party_identifier: Optional[
-            ON_ADD_USER_THIRD_PARTY_IDENTIFIER_CALLBACK
-        ] = None,
-        on_remove_user_third_party_identifier: Optional[
-            ON_REMOVE_USER_THIRD_PARTY_IDENTIFIER_CALLBACK
-        ] = None,
+        check_event_allowed: CHECK_EVENT_ALLOWED_CALLBACK | None = None,
+        on_create_room: ON_CREATE_ROOM_CALLBACK | None = None,
+        check_threepid_can_be_invited: CHECK_THREEPID_CAN_BE_INVITED_CALLBACK
+        | None = None,
+        check_visibility_can_be_modified: CHECK_VISIBILITY_CAN_BE_MODIFIED_CALLBACK
+        | None = None,
+        on_new_event: ON_NEW_EVENT_CALLBACK | None = None,
+        check_can_shutdown_room: CHECK_CAN_SHUTDOWN_ROOM_CALLBACK | None = None,
+        check_can_deactivate_user: CHECK_CAN_DEACTIVATE_USER_CALLBACK | None = None,
+        on_profile_update: ON_PROFILE_UPDATE_CALLBACK | None = None,
+        on_user_deactivation_status_changed: ON_USER_DEACTIVATION_STATUS_CHANGED_CALLBACK
+        | None = None,
+        on_threepid_bind: ON_THREEPID_BIND_CALLBACK | None = None,
+        on_add_user_third_party_identifier: ON_ADD_USER_THIRD_PARTY_IDENTIFIER_CALLBACK
+        | None = None,
+        on_remove_user_third_party_identifier: ON_REMOVE_USER_THIRD_PARTY_IDENTIFIER_CALLBACK
+        | None = None,
     ) -> None:
         """Registers callbacks for third party event rules capabilities.
 
@@ -541,8 +521,8 @@ class ModuleApi:
     def register_presence_router_callbacks(
         self,
         *,
-        get_users_for_states: Optional[GET_USERS_FOR_STATES_CALLBACK] = None,
-        get_interested_users: Optional[GET_INTERESTED_USERS_CALLBACK] = None,
+        get_users_for_states: GET_USERS_FOR_STATES_CALLBACK | None = None,
+        get_interested_users: GET_INTERESTED_USERS_CALLBACK | None = None,
     ) -> None:
         """Registers callbacks for presence router capabilities.
 
@@ -556,18 +536,15 @@ class ModuleApi:
     def register_password_auth_provider_callbacks(
         self,
         *,
-        check_3pid_auth: Optional[CHECK_3PID_AUTH_CALLBACK] = None,
-        on_logged_out: Optional[ON_LOGGED_OUT_CALLBACK] = None,
-        auth_checkers: Optional[
-            Dict[Tuple[str, Tuple[str, ...]], CHECK_AUTH_CALLBACK]
-        ] = None,
-        is_3pid_allowed: Optional[IS_3PID_ALLOWED_CALLBACK] = None,
-        get_username_for_registration: Optional[
-            GET_USERNAME_FOR_REGISTRATION_CALLBACK
-        ] = None,
-        get_displayname_for_registration: Optional[
-            GET_DISPLAYNAME_FOR_REGISTRATION_CALLBACK
-        ] = None,
+        check_3pid_auth: CHECK_3PID_AUTH_CALLBACK | None = None,
+        on_logged_out: ON_LOGGED_OUT_CALLBACK | None = None,
+        auth_checkers: dict[tuple[str, tuple[str, ...]], CHECK_AUTH_CALLBACK]
+        | None = None,
+        is_3pid_allowed: IS_3PID_ALLOWED_CALLBACK | None = None,
+        get_username_for_registration: GET_USERNAME_FOR_REGISTRATION_CALLBACK
+        | None = None,
+        get_displayname_for_registration: GET_DISPLAYNAME_FOR_REGISTRATION_CALLBACK
+        | None = None,
     ) -> None:
         """Registers callbacks for password auth provider capabilities.
 
@@ -591,8 +568,8 @@ class ModuleApi:
         self,
         *,
         on_update: ON_UPDATE_CALLBACK,
-        default_batch_size: Optional[DEFAULT_BATCH_SIZE_CALLBACK] = None,
-        min_batch_size: Optional[MIN_BATCH_SIZE_CALLBACK] = None,
+        default_batch_size: DEFAULT_BATCH_SIZE_CALLBACK | None = None,
+        min_batch_size: MIN_BATCH_SIZE_CALLBACK | None = None,
     ) -> None:
         """Registers background update controller callbacks.
 
@@ -609,7 +586,7 @@ class ModuleApi:
     def register_account_data_callbacks(
         self,
         *,
-        on_account_data_updated: Optional[ON_ACCOUNT_DATA_UPDATED_CALLBACK] = None,
+        on_account_data_updated: ON_ACCOUNT_DATA_UPDATED_CALLBACK | None = None,
     ) -> None:
         """Registers account data callbacks.
 
@@ -638,9 +615,8 @@ class ModuleApi:
     def register_add_extra_fields_to_unsigned_client_event_callbacks(
         self,
         *,
-        add_field_to_unsigned_callback: Optional[
-            ADD_EXTRA_FIELDS_TO_UNSIGNED_CLIENT_EVENT_CALLBACK
-        ] = None,
+        add_field_to_unsigned_callback: ADD_EXTRA_FIELDS_TO_UNSIGNED_CLIENT_EVENT_CALLBACK
+        | None = None,
     ) -> None:
         """Registers a callback that can be used to add fields to the unsigned
         section of events.
@@ -711,7 +687,7 @@ class ModuleApi:
         return self._server_name
 
     @property
-    def worker_name(self) -> Optional[str]:
+    def worker_name(self) -> str | None:
         """The name of the worker this specific instance is running as per the
         "worker_name" configuration setting, or None if it's the main process.
 
@@ -720,7 +696,7 @@ class ModuleApi:
         return self._hs.config.worker.worker_name
 
     @property
-    def worker_app(self) -> Optional[str]:
+    def worker_app(self) -> str | None:
         """The name of the worker app this specific instance is running as per the
         "worker_app" configuration setting, or None if it's the main process.
 
@@ -728,7 +704,7 @@ class ModuleApi:
         """
         return self._hs.config.worker.worker_app
 
-    async def get_userinfo_by_id(self, user_id: str) -> Optional[UserInfo]:
+    async def get_userinfo_by_id(self, user_id: str) -> UserInfo | None:
         """Get user info by user_id
 
         Added in Synapse v1.41.0.
@@ -829,7 +805,7 @@ class ModuleApi:
         user_id = UserID.from_string(f"@{localpart}:{server_name}")
         return await self._store.get_profileinfo(user_id)
 
-    async def get_threepids_for_user(self, user_id: str) -> List[Dict[str, str]]:
+    async def get_threepids_for_user(self, user_id: str) -> list[dict[str, str]]:
         """Look up the threepids (email addresses and phone numbers) associated with the
         given Matrix user ID.
 
@@ -846,7 +822,7 @@ class ModuleApi:
         """
         return [attr.asdict(t) for t in await self._store.user_get_threepids(user_id)]
 
-    def check_user_exists(self, user_id: str) -> "defer.Deferred[Optional[str]]":
+    def check_user_exists(self, user_id: str) -> "defer.Deferred[str | None]":
         """Check if user exists.
 
         Added in Synapse v0.25.0.
@@ -864,9 +840,9 @@ class ModuleApi:
     def register(
         self,
         localpart: str,
-        displayname: Optional[str] = None,
-        emails: Optional[List[str]] = None,
-    ) -> Generator["defer.Deferred[Any]", Any, Tuple[str, str]]:
+        displayname: str | None = None,
+        emails: list[str] | None = None,
+    ) -> Generator["defer.Deferred[Any]", Any, tuple[str, str]]:
         """Registers a new user with given localpart and optional displayname, emails.
 
         Also returns an access token for the new user.
@@ -895,8 +871,8 @@ class ModuleApi:
     def register_user(
         self,
         localpart: str,
-        displayname: Optional[str] = None,
-        emails: Optional[List[str]] = None,
+        displayname: str | None = None,
+        emails: list[str] | None = None,
         admin: bool = False,
     ) -> "defer.Deferred[str]":
         """Registers a new user with given localpart and optional displayname, emails.
@@ -929,9 +905,9 @@ class ModuleApi:
     def register_device(
         self,
         user_id: str,
-        device_id: Optional[str] = None,
-        initial_display_name: Optional[str] = None,
-    ) -> "defer.Deferred[Tuple[str, str, Optional[int], Optional[str]]]":
+        device_id: str | None = None,
+        initial_display_name: str | None = None,
+    ) -> "defer.Deferred[tuple[str, str, int | None, str | None]]":
         """Register a device for a user and generate an access token.
 
         Added in Synapse v1.2.0.
@@ -981,8 +957,8 @@ class ModuleApi:
         self,
         user_id: str,
         duration_in_ms: int = (2 * 60 * 1000),
-        auth_provider_id: Optional[str] = None,
-        auth_provider_session_id: Optional[str] = None,
+        auth_provider_id: str | None = None,
+        auth_provider_session_id: str | None = None,
     ) -> str:
         """Create a login token suitable for m.login.token authentication
 
@@ -1085,7 +1061,7 @@ class ModuleApi:
         )
 
     async def invalidate_cache(
-        self, cached_func: CachedFunction, keys: Tuple[Any, ...]
+        self, cached_func: CachedFunction, keys: tuple[Any, ...]
     ) -> None:
         """Invalidate a cache entry of a cached function across workers. The cached function
         needs to be registered on all workers first with `register_cached_function`.
@@ -1138,7 +1114,7 @@ class ModuleApi:
 
     @defer.inlineCallbacks
     def get_state_events_in_room(
-        self, room_id: str, types: Iterable[Tuple[str, Optional[str]]]
+        self, room_id: str, types: Iterable[tuple[str, str | None]]
     ) -> Generator[defer.Deferred, Any, Iterable[EventBase]]:
         """Gets current state events for the given room.
 
@@ -1169,8 +1145,8 @@ class ModuleApi:
         target: str,
         room_id: str,
         new_membership: str,
-        content: Optional[JsonDict] = None,
-        remote_room_hosts: Optional[List[str]] = None,
+        content: JsonDict | None = None,
+        remote_room_hosts: list[str] | None = None,
     ) -> EventBase:
         """Updates the membership of a user to the given value.
 
@@ -1346,7 +1322,7 @@ class ModuleApi:
             )
 
     async def set_presence_for_users(
-        self, users: Mapping[str, Tuple[str, Optional[str]]]
+        self, users: Mapping[str, tuple[str, str | None]]
     ) -> None:
         """
         Update the internal presence state of users.
@@ -1381,7 +1357,7 @@ class ModuleApi:
         f: Callable,
         msec: float,
         *args: object,
-        desc: Optional[str] = None,
+        desc: str | None = None,
         run_on_all_instances: bool = False,
         **kwargs: object,
     ) -> None:
@@ -1440,7 +1416,7 @@ class ModuleApi:
         msec: float,
         f: Callable,
         *args: object,
-        desc: Optional[str] = None,
+        desc: str | None = None,
         **kwargs: object,
     ) -> IDelayedCall:
         """Wraps a function as a background process and calls it in a given number of milliseconds.
@@ -1486,11 +1462,11 @@ class ModuleApi:
     async def send_http_push_notification(
         self,
         user_id: str,
-        device_id: Optional[str],
+        device_id: str | None,
         content: JsonDict,
-        tweaks: Optional[JsonMapping] = None,
-        default_payload: Optional[JsonMapping] = None,
-    ) -> Dict[str, bool]:
+        tweaks: JsonMapping | None = None,
+        default_payload: JsonMapping | None = None,
+    ) -> dict[str, bool]:
         """Send an HTTP push notification that is forwarded to the registered push gateway
         for the specified user/device.
 
@@ -1554,9 +1530,9 @@ class ModuleApi:
 
     def read_templates(
         self,
-        filenames: List[str],
-        custom_template_directory: Optional[str] = None,
-    ) -> List[jinja2.Template]:
+        filenames: list[str],
+        custom_template_directory: str | None = None,
+    ) -> list[jinja2.Template]:
         """Read and load the content of the template files at the given location.
         By default, Synapse will look for these templates in its configured template
         directory, but another directory to search in can be provided.
@@ -1576,7 +1552,7 @@ class ModuleApi:
             (td for td in (self.custom_template_dir, custom_template_directory) if td),
         )
 
-    def is_mine(self, id: Union[str, DomainSpecificString]) -> bool:
+    def is_mine(self, id: str | DomainSpecificString) -> bool:
         """
         Checks whether an ID (user id, room, ...) comes from this homeserver.
 
@@ -1595,7 +1571,7 @@ class ModuleApi:
 
     async def get_user_ip_and_agents(
         self, user_id: str, since_ts: int = 0
-    ) -> List[UserIpAndAgent]:
+    ) -> list[UserIpAndAgent]:
         """
         Return the list of user IPs and agents for a user.
 
@@ -1638,7 +1614,7 @@ class ModuleApi:
     async def get_room_state(
         self,
         room_id: str,
-        event_filter: Optional[Iterable[Tuple[str, Optional[str]]]] = None,
+        event_filter: Iterable[tuple[str, str | None]] | None = None,
     ) -> StateMap[EventBase]:
         """Returns the current state of the given room.
 
@@ -1680,11 +1656,11 @@ class ModuleApi:
     def run_as_background_process(
         self,
         desc: "LiteralString",
-        func: Callable[..., Awaitable[Optional[T]]],
+        func: Callable[..., Awaitable[T | None]],
         *args: Any,
         bg_start_span: bool = True,
         **kwargs: Any,
-    ) -> "defer.Deferred[Optional[T]]":
+    ) -> "defer.Deferred[T | None]":
         """Run the given function in its own logcontext, with resource metrics
 
         This should be used to wrap processes which are fired off to run in the
@@ -1802,9 +1778,7 @@ class ModuleApi:
         """
         await self._store.add_user_bound_threepid(user_id, medium, address, id_server)
 
-    def check_push_rule_actions(
-        self, actions: List[Union[str, Dict[str, str]]]
-    ) -> None:
+    def check_push_rule_actions(self, actions: list[str | dict[str, str]]) -> None:
         """Checks if the given push rule actions are valid according to the Matrix
         specification.
 
@@ -1827,7 +1801,7 @@ class ModuleApi:
         scope: str,
         kind: str,
         rule_id: str,
-        actions: List[Union[str, Dict[str, str]]],
+        actions: list[str | dict[str, str]],
     ) -> None:
         """Changes the actions of an existing push rule for the given user.
 
@@ -1865,8 +1839,8 @@ class ModuleApi:
         )
 
     async def get_monthly_active_users_by_service(
-        self, start_timestamp: Optional[int] = None, end_timestamp: Optional[int] = None
-    ) -> List[Tuple[str, str]]:
+        self, start_timestamp: int | None = None, end_timestamp: int | None = None
+    ) -> list[tuple[str, str]]:
         """Generates list of monthly active users and their services.
         Please see corresponding storage docstring for more details.
 
@@ -1886,7 +1860,7 @@ class ModuleApi:
             start_timestamp, end_timestamp
         )
 
-    async def get_canonical_room_alias(self, room_id: RoomID) -> Optional[RoomAlias]:
+    async def get_canonical_room_alias(self, room_id: RoomID) -> RoomAlias | None:
         """
         Retrieve the given room's current canonical alias.
 
@@ -1912,7 +1886,7 @@ class ModuleApi:
             return RoomAlias.from_string(room_alias_str)
         return None
 
-    async def lookup_room_alias(self, room_alias: str) -> Tuple[str, List[str]]:
+    async def lookup_room_alias(self, room_alias: str) -> tuple[str, list[str]]:
         """
         Get the room ID associated with a room alias.
 
@@ -1941,8 +1915,8 @@ class ModuleApi:
         user_id: str,
         config: JsonDict,
         ratelimit: bool = True,
-        creator_join_profile: Optional[JsonDict] = None,
-    ) -> Tuple[str, Optional[str]]:
+        creator_join_profile: JsonDict | None = None,
+    ) -> tuple[str, str | None]:
         """Creates a new room.
 
         Added in Synapse v1.65.0.
@@ -2112,7 +2086,7 @@ class AccountDataManager:
                 f"{user_id} is not local to this homeserver; can't access account data for remote users."
             )
 
-    async def get_global(self, user_id: str, data_type: str) -> Optional[JsonMapping]:
+    async def get_global(self, user_id: str, data_type: str) -> JsonMapping | None:
         """
         Gets some global account data, of a specified type, for the specified user.
 
