@@ -23,11 +23,12 @@ protocols.
 
 An explanation of this protocol is available in docs/tcp_replication.md
 """
+
 import fcntl
 import logging
 import struct
 from inspect import isawaitable
-from typing import TYPE_CHECKING, Any, Collection, List, Optional
+from typing import TYPE_CHECKING, Any, Collection
 
 from prometheus_client import Counter
 from zope.interface import Interface, implementer
@@ -38,10 +39,9 @@ from twisted.protocols.basic import LineOnlyReceiver
 from twisted.python.failure import Failure
 
 from synapse.logging.context import PreserveLoggingContext
-from synapse.metrics import LaterGauge
+from synapse.metrics import SERVER_NAME_LABEL, LaterGauge
 from synapse.metrics.background_process_metrics import (
     BackgroundProcessLoggingContext,
-    run_as_background_process,
 )
 from synapse.replication.tcp.commands import (
     VALID_CLIENT_COMMANDS,
@@ -54,7 +54,7 @@ from synapse.replication.tcp.commands import (
     ServerCommand,
     parse_command_from_line,
 )
-from synapse.util import Clock
+from synapse.util.clock import Clock
 from synapse.util.stringutils import random_string
 
 if TYPE_CHECKING:
@@ -63,24 +63,26 @@ if TYPE_CHECKING:
 
 
 connection_close_counter = Counter(
-    "synapse_replication_tcp_protocol_close_reason", "", ["reason_type"]
+    "synapse_replication_tcp_protocol_close_reason",
+    "",
+    labelnames=["reason_type", SERVER_NAME_LABEL],
 )
 
 tcp_inbound_commands_counter = Counter(
     "synapse_replication_tcp_protocol_inbound_commands",
     "Number of commands received from replication, by command and name of process connected to",
-    ["command", "name"],
+    labelnames=["command", "name", SERVER_NAME_LABEL],
 )
 
 tcp_outbound_commands_counter = Counter(
     "synapse_replication_tcp_protocol_outbound_commands",
     "Number of commands sent to replication, by command and name of process connected to",
-    ["command", "name"],
+    labelnames=["command", "name", SERVER_NAME_LABEL],
 )
 
 # A list of all connected protocols. This allows us to send metrics about the
 # connections.
-connected_connections: "List[BaseReplicationStreamProtocol]" = []
+connected_connections: "list[BaseReplicationStreamProtocol]" = []
 
 
 logger = logging.getLogger(__name__)
@@ -136,14 +138,22 @@ class BaseReplicationStreamProtocol(LineOnlyReceiver):
 
     max_line_buffer = 10000
 
-    def __init__(self, clock: Clock, handler: "ReplicationCommandHandler"):
+    def __init__(
+        self,
+        hs: "HomeServer",
+        server_name: str,
+        clock: Clock,
+        handler: "ReplicationCommandHandler",
+    ):
+        self.server_name = server_name
+        self.hs = hs
         self.clock = clock
         self.command_handler = handler
 
         self.last_received_command = self.clock.time_msec()
         self.last_sent_command = 0
         # When we requested the connection be closed
-        self.time_we_closed: Optional[int] = None
+        self.time_we_closed: int | None = None
 
         self.received_ping = False  # Have we received a ping from the other side
 
@@ -153,10 +163,10 @@ class BaseReplicationStreamProtocol(LineOnlyReceiver):
         self.conn_id = random_string(5)  # To dedupe in case of name clashes.
 
         # List of pending commands to send once we've established the connection
-        self.pending_commands: List[Command] = []
+        self.pending_commands: list[Command] = []
 
         # The LoopingCall for sending pings.
-        self._send_ping_loop: Optional[task.LoopingCall] = None
+        self._send_ping_loop: task.LoopingCall | None = None
 
         # a logcontext which we use for processing incoming commands. We declare it as a
         # background process so that the CPU stats get reported to prometheus.
@@ -165,7 +175,9 @@ class BaseReplicationStreamProtocol(LineOnlyReceiver):
             # capture the sentinel context as its containing context and won't prevent
             # GC of / unintentionally reactivate what would be the current context.
             self._logging_context = BackgroundProcessLoggingContext(
-                "replication-conn", self.conn_id
+                name="replication-conn",
+                server_name=self.server_name,
+                instance_id=self.conn_id,
             )
 
     def connectionMade(self) -> None:
@@ -243,7 +255,11 @@ class BaseReplicationStreamProtocol(LineOnlyReceiver):
 
         self.last_received_command = self.clock.time_msec()
 
-        tcp_inbound_commands_counter.labels(cmd.NAME, self.name).inc()
+        tcp_inbound_commands_counter.labels(
+            command=cmd.NAME,
+            name=self.name,
+            **{SERVER_NAME_LABEL: self.server_name},
+        ).inc()
 
         self.handle_command(cmd)
 
@@ -278,8 +294,9 @@ class BaseReplicationStreamProtocol(LineOnlyReceiver):
             # if so.
 
             if isawaitable(res):
-                run_as_background_process(
-                    "replication-" + cmd.get_logcontext_id(), lambda: res
+                self.hs.run_as_background_process(
+                    "replication-" + cmd.get_logcontext_id(),
+                    lambda: res,
                 )
 
             handled = True
@@ -317,7 +334,11 @@ class BaseReplicationStreamProtocol(LineOnlyReceiver):
             self._queue_command(cmd)
             return
 
-        tcp_outbound_commands_counter.labels(cmd.NAME, self.name).inc()
+        tcp_outbound_commands_counter.labels(
+            command=cmd.NAME,
+            name=self.name,
+            **{SERVER_NAME_LABEL: self.server_name},
+        ).inc()
 
         string = "%s %s" % (cmd.NAME, cmd.to_line())
         if "\n" in string:
@@ -389,9 +410,15 @@ class BaseReplicationStreamProtocol(LineOnlyReceiver):
         logger.info("[%s] Replication connection closed: %r", self.id(), reason)
         if isinstance(reason, Failure):
             assert reason.type is not None
-            connection_close_counter.labels(reason.type.__name__).inc()
+            connection_close_counter.labels(
+                reason_type=reason.type.__name__,
+                **{SERVER_NAME_LABEL: self.server_name},
+            ).inc()
         else:
-            connection_close_counter.labels(reason.__class__.__name__).inc()  # type: ignore[unreachable]
+            connection_close_counter.labels(  # type: ignore[unreachable]
+                reason_type=reason.__class__.__name__,
+                **{SERVER_NAME_LABEL: self.server_name},
+            ).inc()
 
         try:
             # Remove us from list of connections to be monitored
@@ -446,9 +473,13 @@ class ServerReplicationStreamProtocol(BaseReplicationStreamProtocol):
     VALID_OUTBOUND_COMMANDS = VALID_SERVER_COMMANDS
 
     def __init__(
-        self, server_name: str, clock: Clock, handler: "ReplicationCommandHandler"
+        self,
+        hs: "HomeServer",
+        server_name: str,
+        clock: Clock,
+        handler: "ReplicationCommandHandler",
     ):
-        super().__init__(clock, handler)
+        super().__init__(hs, server_name, clock, handler)
 
         self.server_name = server_name
 
@@ -473,7 +504,7 @@ class ClientReplicationStreamProtocol(BaseReplicationStreamProtocol):
         clock: Clock,
         command_handler: "ReplicationCommandHandler",
     ):
-        super().__init__(clock, command_handler)
+        super().__init__(hs, server_name, clock, command_handler)
 
         self.client_name = client_name
         self.server_name = server_name
@@ -500,10 +531,15 @@ class ClientReplicationStreamProtocol(BaseReplicationStreamProtocol):
 # The following simply registers metrics for the replication connections
 
 pending_commands = LaterGauge(
-    "synapse_replication_tcp_protocol_pending_commands",
-    "",
-    ["name"],
-    lambda: {(p.name,): len(p.pending_commands) for p in connected_connections},
+    name="synapse_replication_tcp_protocol_pending_commands",
+    desc="",
+    labelnames=["name", SERVER_NAME_LABEL],
+)
+pending_commands.register_hook(
+    homeserver_instance_id=None,
+    hook=lambda: {
+        (p.name, p.server_name): len(p.pending_commands) for p in connected_connections
+    },
 )
 
 
@@ -515,10 +551,15 @@ def transport_buffer_size(protocol: BaseReplicationStreamProtocol) -> int:
 
 
 transport_send_buffer = LaterGauge(
-    "synapse_replication_tcp_protocol_transport_send_buffer",
-    "",
-    ["name"],
-    lambda: {(p.name,): transport_buffer_size(p) for p in connected_connections},
+    name="synapse_replication_tcp_protocol_transport_send_buffer",
+    desc="",
+    labelnames=["name", SERVER_NAME_LABEL],
+)
+transport_send_buffer.register_hook(
+    homeserver_instance_id=None,
+    hook=lambda: {
+        (p.name, p.server_name): transport_buffer_size(p) for p in connected_connections
+    },
 )
 
 
@@ -540,22 +581,28 @@ def transport_kernel_read_buffer_size(
 
 
 tcp_transport_kernel_send_buffer = LaterGauge(
-    "synapse_replication_tcp_protocol_transport_kernel_send_buffer",
-    "",
-    ["name"],
-    lambda: {
-        (p.name,): transport_kernel_read_buffer_size(p, False)
+    name="synapse_replication_tcp_protocol_transport_kernel_send_buffer",
+    desc="",
+    labelnames=["name", SERVER_NAME_LABEL],
+)
+tcp_transport_kernel_send_buffer.register_hook(
+    homeserver_instance_id=None,
+    hook=lambda: {
+        (p.name, p.server_name): transport_kernel_read_buffer_size(p, False)
         for p in connected_connections
     },
 )
 
 
 tcp_transport_kernel_read_buffer = LaterGauge(
-    "synapse_replication_tcp_protocol_transport_kernel_read_buffer",
-    "",
-    ["name"],
-    lambda: {
-        (p.name,): transport_kernel_read_buffer_size(p, True)
+    name="synapse_replication_tcp_protocol_transport_kernel_read_buffer",
+    desc="",
+    labelnames=["name", SERVER_NAME_LABEL],
+)
+tcp_transport_kernel_read_buffer.register_hook(
+    homeserver_instance_id=None,
+    hook=lambda: {
+        (p.name, p.server_name): transport_kernel_read_buffer_size(p, True)
         for p in connected_connections
     },
 )
