@@ -22,13 +22,8 @@
 import logging
 from typing import (
     TYPE_CHECKING,
-    Dict,
-    List,
     Mapping,
-    Optional,
-    Tuple,
     TypedDict,
-    Union,
     cast,
 )
 
@@ -67,9 +62,9 @@ class DeviceLastConnectionInfo:
     user_id: str
     device_id: str
 
-    ip: Optional[str]
-    user_agent: Optional[str]
-    last_seen: Optional[int]
+    ip: str | None
+    user_agent: str | None
+    last_seen: int | None
 
 
 class LastConnectionInfo(TypedDict):
@@ -179,7 +174,7 @@ class ClientIpBackgroundUpdateStore(SQLBaseStore):
         # Fetch the start of the batch
         begin_last_seen: int = progress.get("last_seen", 0)
 
-        def get_last_seen(txn: LoggingTransaction) -> Optional[int]:
+        def get_last_seen(txn: LoggingTransaction) -> int | None:
             txn.execute(
                 """
                 SELECT last_seen FROM user_ips
@@ -190,7 +185,7 @@ class ClientIpBackgroundUpdateStore(SQLBaseStore):
                 """,
                 (begin_last_seen, batch_size),
             )
-            row = cast(Optional[Tuple[int]], txn.fetchone())
+            row = cast(tuple[int] | None, txn.fetchone())
             if row:
                 return row[0]
             else:
@@ -222,7 +217,7 @@ class ClientIpBackgroundUpdateStore(SQLBaseStore):
 
             # Define the search space, which requires handling the last batch in
             # a different way
-            args: Tuple[int, ...]
+            args: tuple[int, ...]
             if last:
                 clause = "? <= last_seen"
                 args = (begin_last_seen,)
@@ -251,7 +246,7 @@ class ClientIpBackgroundUpdateStore(SQLBaseStore):
                 args,
             )
             res = cast(
-                List[Tuple[str, str, str, Optional[str], str, int, int]], txn.fetchall()
+                list[tuple[str, str, str, str | None, str, int, int]], txn.fetchall()
             )
 
             # We've got some duplicates
@@ -361,7 +356,7 @@ class ClientIpBackgroundUpdateStore(SQLBaseStore):
             #      we'll just end up updating the same device row multiple
             #      times, which is fine.
 
-            where_args: List[Union[str, int]]
+            where_args: list[str | int]
             where_clause, where_args = make_tuple_comparison_clause(
                 [("user_id", last_user_id), ("device_id", last_device_id)],
             )
@@ -383,7 +378,7 @@ class ClientIpBackgroundUpdateStore(SQLBaseStore):
             """ % {"where_clause": where_clause}
             txn.execute(sql, where_args + [batch_size])
 
-            rows = cast(List[Tuple[int, str, str, str, str]], txn.fetchall())
+            rows = cast(list[tuple[int, str, str, str, str]], txn.fetchall())
             if not rows:
                 return 0
 
@@ -434,29 +429,30 @@ class ClientIpWorkerStore(ClientIpBackgroundUpdateStore, MonthlyActiveUsersWorke
         self.user_ips_max_age = hs.config.server.user_ips_max_age
 
         # (user_id, access_token, ip,) -> last_seen
-        self.client_ip_last_seen = LruCache[Tuple[str, str, str], int](
+        self.client_ip_last_seen = LruCache[tuple[str, str, str], int](
             cache_name="client_ip_last_seen",
             server_name=self.server_name,
             max_size=50000,
+            clock=hs.get_clock(),
         )
 
         if hs.config.worker.run_background_tasks and self.user_ips_max_age:
-            self._clock.looping_call(self._prune_old_user_ips, 5 * 1000)
+            self.clock.looping_call(self._prune_old_user_ips, 5 * 1000)
 
         if self._update_on_this_worker:
             # This is the designated worker that can write to the client IP
             # tables.
 
             # (user_id, access_token, ip,) -> (user_agent, device_id, last_seen)
-            self._batch_row_update: Dict[
-                Tuple[str, str, str], Tuple[str, Optional[str], int]
+            self._batch_row_update: dict[
+                tuple[str, str, str], tuple[str, str | None, int]
             ] = {}
 
-            self._client_ip_looper = self._clock.looping_call(
-                self._update_client_ips_batch, 5 * 1000
-            )
-            self.hs.get_clock().add_system_event_trigger(
-                "before", "shutdown", self._update_client_ips_batch
+            self.clock.looping_call(self._update_client_ips_batch, 5 * 1000)
+            hs.register_async_shutdown_handler(
+                phase="before",
+                eventType="shutdown",
+                shutdown_func=self._update_client_ips_batch,
             )
 
     @wrap_as_background_process("prune_old_user_ips")
@@ -492,7 +488,7 @@ class ClientIpWorkerStore(ClientIpBackgroundUpdateStore, MonthlyActiveUsersWorke
             )
         """
 
-        timestamp = self._clock.time_msec() - self.user_ips_max_age
+        timestamp = self.clock.time_msec() - self.user_ips_max_age
 
         def _prune_old_user_ips_txn(txn: LoggingTransaction) -> None:
             txn.execute(sql, (timestamp,))
@@ -502,8 +498,8 @@ class ClientIpWorkerStore(ClientIpBackgroundUpdateStore, MonthlyActiveUsersWorke
         )
 
     async def _get_last_client_ip_by_device_from_database(
-        self, user_id: str, device_id: Optional[str]
-    ) -> Dict[Tuple[str, str], DeviceLastConnectionInfo]:
+        self, user_id: str, device_id: str | None
+    ) -> dict[tuple[str, str], DeviceLastConnectionInfo]:
         """For each device_id listed, give the user_ip it was last seen on.
 
         The result might be slightly out of date as client IPs are inserted in batches.
@@ -521,7 +517,7 @@ class ClientIpWorkerStore(ClientIpBackgroundUpdateStore, MonthlyActiveUsersWorke
             keyvalues["device_id"] = device_id
 
         res = cast(
-            List[Tuple[str, Optional[str], Optional[str], str, Optional[int]]],
+            list[tuple[str, str | None, str | None, str, int | None]],
             await self.db_pool.simple_select_list(
                 table="devices",
                 keyvalues=keyvalues,
@@ -542,7 +538,7 @@ class ClientIpWorkerStore(ClientIpBackgroundUpdateStore, MonthlyActiveUsersWorke
 
     async def _get_user_ip_and_agents_from_database(
         self, user: UserID, since_ts: int = 0
-    ) -> List[LastConnectionInfo]:
+    ) -> list[LastConnectionInfo]:
         """Fetch the IPs and user agents for a user since the given timestamp.
 
         The result might be slightly out of date as client IPs are inserted in batches.
@@ -566,7 +562,7 @@ class ClientIpWorkerStore(ClientIpBackgroundUpdateStore, MonthlyActiveUsersWorke
         """
         user_id = user.to_string()
 
-        def get_recent(txn: LoggingTransaction) -> List[Tuple[str, str, str, int]]:
+        def get_recent(txn: LoggingTransaction) -> list[tuple[str, str, str, int]]:
             txn.execute(
                 """
                 SELECT access_token, ip, user_agent, last_seen FROM user_ips
@@ -576,7 +572,7 @@ class ClientIpWorkerStore(ClientIpBackgroundUpdateStore, MonthlyActiveUsersWorke
                 """,
                 (since_ts, user_id),
             )
-            return cast(List[Tuple[str, str, str, int]], txn.fetchall())
+            return cast(list[tuple[str, str, str, int]], txn.fetchall())
 
         rows = await self.db_pool.runInteraction(
             desc="get_user_ip_and_agents", func=get_recent
@@ -598,8 +594,8 @@ class ClientIpWorkerStore(ClientIpBackgroundUpdateStore, MonthlyActiveUsersWorke
         access_token: str,
         ip: str,
         user_agent: str,
-        device_id: Optional[str],
-        now: Optional[int] = None,
+        device_id: str | None,
+        now: int | None = None,
     ) -> None:
         """Record that `user_id` used `access_token` from this `ip` address.
 
@@ -628,7 +624,7 @@ class ClientIpWorkerStore(ClientIpBackgroundUpdateStore, MonthlyActiveUsersWorke
             return
 
         if not now:
-            now = int(self._clock.time_msec())
+            now = int(self.clock.time_msec())
         key = (user_id, access_token, ip)
 
         try:
@@ -672,7 +668,7 @@ class ClientIpWorkerStore(ClientIpBackgroundUpdateStore, MonthlyActiveUsersWorke
     def _update_client_ips_batch_txn(
         self,
         txn: LoggingTransaction,
-        to_update: Mapping[Tuple[str, str, str], Tuple[str, Optional[str], int]],
+        to_update: Mapping[tuple[str, str, str], tuple[str, str | None, int]],
     ) -> None:
         assert self._update_on_this_worker, (
             "This worker is not designated to update client IPs"
@@ -717,8 +713,8 @@ class ClientIpWorkerStore(ClientIpBackgroundUpdateStore, MonthlyActiveUsersWorke
             )
 
     async def get_last_client_ip_by_device(
-        self, user_id: str, device_id: Optional[str]
-    ) -> Dict[Tuple[str, str], DeviceLastConnectionInfo]:
+        self, user_id: str, device_id: str | None
+    ) -> dict[tuple[str, str], DeviceLastConnectionInfo]:
         """For each device_id listed, give the user_ip it was last seen on
 
         Args:
@@ -758,7 +754,7 @@ class ClientIpWorkerStore(ClientIpBackgroundUpdateStore, MonthlyActiveUsersWorke
 
     async def get_user_ip_and_agents(
         self, user: UserID, since_ts: int = 0
-    ) -> List[LastConnectionInfo]:
+    ) -> list[LastConnectionInfo]:
         """Fetch the IPs and user agents for a user since the given timestamp.
 
         Args:
@@ -785,7 +781,7 @@ class ClientIpWorkerStore(ClientIpBackgroundUpdateStore, MonthlyActiveUsersWorke
             # the result
             return rows_from_db
 
-        results: Dict[Tuple[str, str], LastConnectionInfo] = {
+        results: dict[tuple[str, str], LastConnectionInfo] = {
             (connection["access_token"], connection["ip"]): connection
             for connection in rows_from_db
         }
@@ -807,7 +803,7 @@ class ClientIpWorkerStore(ClientIpBackgroundUpdateStore, MonthlyActiveUsersWorke
 
         return list(results.values())
 
-    async def get_last_seen_for_user_id(self, user_id: str) -> Optional[int]:
+    async def get_last_seen_for_user_id(self, user_id: str) -> int | None:
         """Get the last seen timestamp for a user, if we have it."""
 
         return await self.db_pool.simple_select_one_onecol(
