@@ -21,12 +21,15 @@
 import logging
 from contextlib import AsyncExitStack
 from types import TracebackType
-from typing import TYPE_CHECKING, Collection, Optional, Set, Tuple, Type
+from typing import TYPE_CHECKING, Collection, Optional
 from weakref import WeakValueDictionary
 
+from twisted.internet import defer
 from twisted.internet.task import LoopingCall
 
-from synapse.metrics.background_process_metrics import wrap_as_background_process
+from synapse.metrics.background_process_metrics import (
+    wrap_as_background_process,
+)
 from synapse.storage._base import SQLBaseStore
 from synapse.storage.database import (
     DatabasePool,
@@ -34,7 +37,7 @@ from synapse.storage.database import (
     LoggingTransaction,
 )
 from synapse.types import ISynapseReactor
-from synapse.util import Clock
+from synapse.util.clock import Clock
 from synapse.util.stringutils import random_string
 
 if TYPE_CHECKING:
@@ -79,31 +82,31 @@ class LockStore(SQLBaseStore):
 
         # A map from `(lock_name, lock_key)` to lock that we think we
         # currently hold.
-        self._live_lock_tokens: WeakValueDictionary[
-            Tuple[str, str], Lock
-        ] = WeakValueDictionary()
+        self._live_lock_tokens: WeakValueDictionary[tuple[str, str], Lock] = (
+            WeakValueDictionary()
+        )
 
         # A map from `(lock_name, lock_key, token)` to read/write lock that we
         # think we currently hold. For a given lock_name/lock_key, there can be
         # multiple read locks at a time but only one write lock (no mixing read
         # and write locks at the same time).
         self._live_read_write_lock_tokens: WeakValueDictionary[
-            Tuple[str, str, str], Lock
+            tuple[str, str, str], Lock
         ] = WeakValueDictionary()
 
         # When we shut down we want to remove the locks. Technically this can
         # lead to a race, as we may drop the lock while we are still processing.
         # However, a) it should be a small window, b) the lock is best effort
         # anyway and c) we want to really avoid leaking locks when we restart.
-        hs.get_reactor().addSystemEventTrigger(
-            "before",
-            "shutdown",
-            self._on_shutdown,
+        hs.register_async_shutdown_handler(
+            phase="before",
+            eventType="shutdown",
+            shutdown_func=self._on_shutdown,
         )
 
-        self._acquiring_locks: Set[Tuple[str, str]] = set()
+        self._acquiring_locks: set[tuple[str, str]] = set()
 
-        self._clock.looping_call(
+        self.clock.looping_call(
             self._reap_stale_read_write_locks, _LOCK_TIMEOUT_MS / 10.0
         )
 
@@ -149,7 +152,7 @@ class LockStore(SQLBaseStore):
         if lock and await lock.is_still_valid():
             return None
 
-        now = self._clock.time_msec()
+        now = self.clock.time_msec()
         token = random_string(6)
 
         def _try_acquire_lock_txn(txn: LoggingTransaction) -> bool:
@@ -196,8 +199,10 @@ class LockStore(SQLBaseStore):
             return None
 
         lock = Lock(
+            self.server_name,
             self._reactor,
-            self._clock,
+            self.hs,
+            self.clock,
             self,
             read_write=False,
             lock_name=lock_name,
@@ -246,7 +251,7 @@ class LockStore(SQLBaseStore):
         # constraints. If it doesn't then we have acquired the lock,
         # otherwise we haven't.
 
-        now = self._clock.time_msec()
+        now = self.clock.time_msec()
         token = random_string(6)
 
         self.db_pool.simple_insert_txn(
@@ -263,8 +268,10 @@ class LockStore(SQLBaseStore):
         )
 
         lock = Lock(
+            self.server_name,
             self._reactor,
-            self._clock,
+            self.hs,
+            self.clock,
             self,
             read_write=True,
             lock_name=lock_name,
@@ -281,9 +288,9 @@ class LockStore(SQLBaseStore):
 
     async def try_acquire_multi_read_write_lock(
         self,
-        lock_names: Collection[Tuple[str, str]],
+        lock_names: Collection[tuple[str, str]],
         write: bool,
-    ) -> Optional[AsyncExitStack]:
+    ) -> AsyncExitStack | None:
         """Try to acquire multiple locks for the given names/keys. Will return
         an async context manager if the locks are successfully acquired, which
         *must* be used (otherwise the lock will leak).
@@ -311,7 +318,7 @@ class LockStore(SQLBaseStore):
     def _try_acquire_multi_read_write_lock_txn(
         self,
         txn: LoggingTransaction,
-        lock_names: Collection[Tuple[str, str]],
+        lock_names: Collection[tuple[str, str]],
         write: bool,
     ) -> Collection["Lock"]:
         locks = []
@@ -332,7 +339,7 @@ class LockStore(SQLBaseStore):
         """
 
         def reap_stale_read_write_locks_txn(txn: LoggingTransaction) -> None:
-            txn.execute(delete_sql, (self._clock.time_msec() - _LOCK_TIMEOUT_MS,))
+            txn.execute(delete_sql, (self.clock.time_msec() - _LOCK_TIMEOUT_MS,))
             if txn.rowcount:
                 logger.info("Reaped %d stale locks", txn.rowcount)
 
@@ -366,7 +373,9 @@ class Lock:
 
     def __init__(
         self,
+        server_name: str,
         reactor: ISynapseReactor,
+        hs: "HomeServer",
         clock: Clock,
         store: LockStore,
         read_write: bool,
@@ -374,7 +383,13 @@ class Lock:
         lock_key: str,
         token: str,
     ) -> None:
+        """
+        Args:
+            server_name: The homeserver name (used to label metrics) (this should be `hs.hostname`).
+        """
+        self._server_name = server_name
         self._reactor = reactor
+        self._hs = hs
         self._clock = clock
         self._store = store
         self._read_write = read_write
@@ -387,7 +402,7 @@ class Lock:
 
         # We might be called from a non-main thread, so we defer setting up the
         # looping call.
-        self._looping_call: Optional[LoopingCall] = None
+        self._looping_call: LoopingCall | None = None
         reactor.callFromThread(self._setup_looping_call)
 
         self._dropped = False
@@ -396,7 +411,9 @@ class Lock:
         self._looping_call = self._clock.looping_call(
             self._renew,
             _RENEWAL_INTERVAL_MS,
+            self._server_name,
             self._store,
+            self._hs,
             self._clock,
             self._read_write,
             self._lock_name,
@@ -405,31 +422,55 @@ class Lock:
         )
 
     @staticmethod
-    @wrap_as_background_process("Lock._renew")
-    async def _renew(
+    def _renew(
+        server_name: str,
         store: LockStore,
+        hs: "HomeServer",
         clock: Clock,
         read_write: bool,
         lock_name: str,
         lock_key: str,
         token: str,
-    ) -> None:
+    ) -> "defer.Deferred[None]":
         """Renew the lock.
 
         Note: this is a static method, rather than using self.*, so that we
         don't end up with a reference to `self` in the reactor, which would stop
         this from being cleaned up if we dropped the context manager.
+
+        Args:
+            server_name: The homeserver name (used to label metrics) (this should be `hs.hostname`).
         """
-        table = "worker_read_write_locks" if read_write else "worker_locks"
-        await store.db_pool.simple_update(
-            table=table,
-            keyvalues={
-                "lock_name": lock_name,
-                "lock_key": lock_key,
-                "token": token,
-            },
-            updatevalues={"last_renewed_ts": clock.time_msec()},
-            desc="renew_lock",
+
+        async def _internal_renew(
+            store: LockStore,
+            clock: Clock,
+            read_write: bool,
+            lock_name: str,
+            lock_key: str,
+            token: str,
+        ) -> None:
+            table = "worker_read_write_locks" if read_write else "worker_locks"
+            await store.db_pool.simple_update(
+                table=table,
+                keyvalues={
+                    "lock_name": lock_name,
+                    "lock_key": lock_key,
+                    "token": token,
+                },
+                updatevalues={"last_renewed_ts": clock.time_msec()},
+                desc="renew_lock",
+            )
+
+        return hs.run_as_background_process(
+            "Lock._renew",
+            _internal_renew,
+            store,
+            clock,
+            read_write,
+            lock_name,
+            lock_key,
+            token,
         )
 
     async def is_still_valid(self) -> bool:
@@ -456,9 +497,9 @@ class Lock:
 
     async def __aexit__(
         self,
-        _exctype: Optional[Type[BaseException]],
-        _excinst: Optional[BaseException],
-        _exctb: Optional[TracebackType],
+        _exctype: type[BaseException] | None,
+        _excinst: BaseException | None,
+        _exctb: TracebackType | None,
     ) -> bool:
         await self.release()
 
