@@ -18,7 +18,6 @@
 # [This file includes modifications made by New Vector Limited]
 #
 #
-from typing import Optional
 from unittest import mock
 
 from twisted.internet.testing import MemoryReactor
@@ -183,7 +182,7 @@ class FederationEventHandlerTests(unittest.FederatingHomeserverTestCase):
         else:
 
             async def get_event(
-                destination: str, event_id: str, timeout: Optional[int] = None
+                destination: str, event_id: str, timeout: int | None = None
             ) -> JsonDict:
                 self.assertEqual(destination, self.OTHER_SERVER_NAME)
                 self.assertEqual(event_id, prev_event.event_id)
@@ -224,7 +223,10 @@ class FederationEventHandlerTests(unittest.FederatingHomeserverTestCase):
         self.reactor.advance(60000)
 
         # Finally, the call under test: send the pulled event into _process_pulled_event
-        with LoggingContext("test"):
+        with LoggingContext(
+            name="test",
+            server_name=self.hs.hostname,
+        ):
             self.get_success(
                 self.hs.get_federation_event_handler()._process_pulled_event(
                     self.OTHER_SERVER_NAME, pulled_event, backfilled=False
@@ -321,7 +323,10 @@ class FederationEventHandlerTests(unittest.FederatingHomeserverTestCase):
         )
 
         # The function under test: try to process the pulled event
-        with LoggingContext("test"):
+        with LoggingContext(
+            name="test",
+            server_name=self.hs.hostname,
+        ):
             self.get_success(
                 self.hs.get_federation_event_handler()._process_pulled_event(
                     self.OTHER_SERVER_NAME, pulled_event, backfilled=True
@@ -339,7 +344,10 @@ class FederationEventHandlerTests(unittest.FederatingHomeserverTestCase):
         self.assertEqual(backfill_num_attempts, 1)
 
         # The function under test: try to process the pulled event again
-        with LoggingContext("test"):
+        with LoggingContext(
+            name="test",
+            server_name=self.hs.hostname,
+        ):
             self.get_success(
                 self.hs.get_federation_event_handler()._process_pulled_event(
                     self.OTHER_SERVER_NAME, pulled_event, backfilled=True
@@ -447,7 +455,10 @@ class FederationEventHandlerTests(unittest.FederatingHomeserverTestCase):
         self.assertEqual(backfill_num_attempts, 1)
 
         # The function under test: try to process the pulled event
-        with LoggingContext("test"):
+        with LoggingContext(
+            name="test",
+            server_name=self.hs.hostname,
+        ):
             self.get_success(
                 self.hs.get_federation_event_handler()._process_pulled_event(
                     self.OTHER_SERVER_NAME, pulled_event, backfilled=True
@@ -573,7 +584,7 @@ class FederationEventHandlerTests(unittest.FederatingHomeserverTestCase):
         room_state_endpoint_requested_count = 0
 
         async def get_event(
-            destination: str, event_id: str, timeout: Optional[int] = None
+            destination: str, event_id: str, timeout: int | None = None
         ) -> None:
             nonlocal event_endpoint_requested_count
             event_endpoint_requested_count += 1
@@ -602,7 +613,10 @@ class FederationEventHandlerTests(unittest.FederatingHomeserverTestCase):
         )
 
         # The function under test: try to backfill and process the pulled event
-        with LoggingContext("test"):
+        with LoggingContext(
+            name="test",
+            server_name=self.hs.hostname,
+        ):
             self.get_success(
                 self.hs.get_federation_event_handler().backfill(
                     self.OTHER_SERVER_NAME,
@@ -742,7 +756,10 @@ class FederationEventHandlerTests(unittest.FederatingHomeserverTestCase):
         }
 
         # The function under test: try to backfill and process the pulled event
-        with LoggingContext("test"):
+        with LoggingContext(
+            name="test",
+            server_name=self.hs.hostname,
+        ):
             self.get_success(
                 self.hs.get_federation_event_handler().backfill(
                     self.OTHER_SERVER_NAME,
@@ -887,7 +904,10 @@ class FederationEventHandlerTests(unittest.FederatingHomeserverTestCase):
         next_depth += 1
         next_timestamp += 100
 
-        with LoggingContext("send_rejected_power_levels_event"):
+        with LoggingContext(
+            name="send_rejected_power_levels_event",
+            server_name=self.hs.hostname,
+        ):
             self.get_success(
                 self.hs.get_federation_event_handler()._process_pulled_event(
                     self.OTHER_SERVER_NAME,
@@ -969,7 +989,10 @@ class FederationEventHandlerTests(unittest.FederatingHomeserverTestCase):
             "during state resolution. The test setup is incorrect.",
         )
 
-        with LoggingContext("send_rejected_kick_event"):
+        with LoggingContext(
+            name="send_rejected_kick_event",
+            server_name=self.hs.hostname,
+        ):
             self.get_success(
                 self.hs.get_federation_event_handler()._process_pulled_event(
                     self.OTHER_SERVER_NAME, rejected_kick_event, backfilled=False
@@ -1085,10 +1108,13 @@ class FederationEventHandlerTests(unittest.FederatingHomeserverTestCase):
         # We have to bump the clock a bit, to keep the retry logic in
         # `FederationClient.get_pdu` happy
         self.reactor.advance(60000)
-        with LoggingContext("send_pulled_event"):
+        with LoggingContext(
+            name="send_pulled_event",
+            server_name=self.hs.hostname,
+        ):
 
             async def get_event(
-                destination: str, event_id: str, timeout: Optional[int] = None
+                destination: str, event_id: str, timeout: int | None = None
             ) -> JsonDict:
                 self.assertEqual(destination, self.OTHER_SERVER_NAME)
                 self.assertEqual(event_id, missing_event.event_id)
