@@ -19,7 +19,7 @@
 #
 #
 
-from typing import Any, Callable, Iterable, Optional
+from typing import Any, Callable, Iterable
 from unittest.mock import Mock
 
 from twisted.internet.testing import MemoryReactor
@@ -56,8 +56,8 @@ class BaseWorkerStoreTestCase(BaseStreamTestCase):
         self,
         method: str,
         args: Iterable[Any],
-        expected_result: Optional[Any] = None,
-        asserter: Optional[Callable[[Any, Any, Optional[Any]], None]] = None,
+        expected_result: Any | None = None,
+        asserter: Callable[[Any, Any, Any | None], None] | None = None,
     ) -> None:
         if asserter is None:
             asserter = self.assertEqual
