@@ -24,7 +24,7 @@ from twisted.internet.testing import MemoryReactor
 
 from synapse.rest.client import register
 from synapse.server import HomeServer
-from synapse.util import Clock
+from synapse.util.clock import Clock
 
 from tests.replication._base import BaseMultiWorkerStreamTestCase
 from tests.server import FakeChannel, make_request
