@@ -18,7 +18,6 @@
 # [This file includes modifications made by New Vector Limited]
 #
 #
-from typing import List, Optional, Tuple
 from unittest.mock import AsyncMock, Mock
 
 from twisted.internet.testing import MemoryReactor
@@ -26,7 +25,7 @@ from twisted.internet.testing import MemoryReactor
 import synapse.rest.admin
 from synapse.rest.client import login, notifications, receipts, room
 from synapse.server import HomeServer
-from synapse.util import Clock
+from synapse.util.clock import Clock
 
 from tests.unittest import HomeserverTestCase
 
@@ -155,8 +154,8 @@ class HTTPPusherTests(HomeserverTestCase):
         self.assertEqual(notification_event_ids, sent_event_ids[2:])
 
     def _request_notifications(
-        self, from_token: Optional[str], limit: int, expected_count: int
-    ) -> Tuple[List[str], str]:
+        self, from_token: str | None, limit: int, expected_count: int
+    ) -> tuple[list[str], str]:
         """
         Make a request to /notifications to get the latest events to be notified about.
 
