@@ -18,7 +18,7 @@
 #
 #
 import itertools
-from typing import Dict, List, Optional, Sequence, Set
+from typing import Sequence
 
 from twisted.internet import defer
 from twisted.test.proto_helpers import MemoryReactor
@@ -38,7 +38,7 @@ from synapse.state.v2 import (
     resolve_events_with_store,
 )
 from synapse.types import StateMap
-from synapse.util import Clock
+from synapse.util.clock import Clock
 
 from tests import unittest
 from tests.state.test_v2 import TestStateResolutionStore
@@ -66,8 +66,8 @@ def monotonic_timestamp() -> int:
 
 
 class FakeClock:
-    def sleep(self, msec: float) -> "defer.Deferred[None]":
-        return defer.succeed(None)
+    async def sleep(self, duration_ms: float) -> None:
+        defer.succeed(None)
 
 
 class StateResV21TestCase(unittest.HomeserverTestCase):
@@ -357,11 +357,11 @@ class StateResV21TestCase(unittest.HomeserverTestCase):
         self,
         room_id: str,
         state_maps: Sequence[StateMap[str]],
-        event_map: Optional[Dict[str, EventBase]],
+        event_map: dict[str, EventBase] | None,
         state_res_store: StateResolutionStoreInterface,
-    ) -> Set[str]:
+    ) -> set[str]:
         _, conflicted_state = _seperate(state_maps)
-        conflicted_set: Optional[Set[str]] = set(
+        conflicted_set: set[str] | None = set(
             itertools.chain.from_iterable(conflicted_state.values())
         )
         if event_map is None:
@@ -377,7 +377,7 @@ class StateResV21TestCase(unittest.HomeserverTestCase):
     def get_resolution_and_verify_expected(
         self,
         state_maps: Sequence[StateMap[str]],
-        events: List[EventBase],
+        events: list[EventBase],
         expected: StateMap[str],
     ) -> None:
         room_id = events[0].room_id
@@ -458,7 +458,7 @@ class StateResV21TestCase(unittest.HomeserverTestCase):
             resolve_and_check()
 
     def persist_event(
-        self, event: EventBase, state: Optional[StateMap[str]] = None
+        self, event: EventBase, state: StateMap[str] | None = None
     ) -> None:
         """Persist the event, with optional state"""
         context = self.get_success(
@@ -473,12 +473,12 @@ class StateResV21TestCase(unittest.HomeserverTestCase):
     def create_event(
         self,
         event_type: str,
-        state_key: Optional[str],
+        state_key: str | None,
         sender: str,
-        content: Dict,
-        auth_events: List[str],
-        prev_events: Optional[List[str]] = None,
-        room_id: Optional[str] = None,
+        content: dict,
+        auth_events: list[str],
+        prev_events: list[str] | None = None,
+        room_id: str | None = None,
     ) -> EventBase:
         """Short-hand for event_from_pdu_json for fields we typically care about.
         Tests can override by just calling event_from_pdu_json directly."""
