@@ -19,7 +19,7 @@
 #
 #
 
-from typing import Any, Dict, Optional, Sequence, Tuple
+from typing import Any, Sequence
 
 from twisted.internet.testing import MemoryReactor
 
@@ -51,8 +51,8 @@ class ReceiptsBackgroundUpdateStoreTestCase(HomeserverTestCase):
         update_name: str,
         index_name: str,
         table: str,
-        receipts: Dict[Tuple[str, str, str], Sequence[Dict[str, Any]]],
-        expected_unique_receipts: Dict[Tuple[str, str, str], Optional[Dict[str, Any]]],
+        receipts: dict[tuple[str, str, str], Sequence[dict[str, Any]]],
+        expected_unique_receipts: dict[tuple[str, str, str], dict[str, Any] | None],
     ) -> None:
         """Test that the background update to uniqueify non-thread receipts in
         the given receipts table works properly.
