@@ -18,9 +18,8 @@
 # [This file includes modifications made by New Vector Limited]
 #
 #
-from typing import Dict, List, Optional
 
-from twisted.test.proto_helpers import MemoryReactor
+from twisted.internet.testing import MemoryReactor
 
 from synapse.server import HomeServer
 from synapse.storage.database import (
@@ -35,7 +34,7 @@ from synapse.storage.util.sequence import (
     PostgresSequenceGenerator,
     SequenceGenerator,
 )
-from synapse.util import Clock
+from synapse.util.clock import Clock
 
 from tests.unittest import HomeserverTestCase
 from tests.utils import USE_POSTGRES_FOR_TESTS
@@ -43,12 +42,12 @@ from tests.utils import USE_POSTGRES_FOR_TESTS
 
 class MultiWriterIdGeneratorBase(HomeserverTestCase):
     positive: bool = True
-    tables: List[str] = ["foobar"]
+    tables: list[str] = ["foobar"]
 
     def prepare(self, reactor: MemoryReactor, clock: Clock, hs: HomeServer) -> None:
         self.store = hs.get_datastores().main
         self.db_pool: DatabasePool = self.store.db_pool
-        self.instances: Dict[str, MultiWriterIdGenerator] = {}
+        self.instances: dict[str, MultiWriterIdGenerator] = {}
 
         self.get_success(self.db_pool.runInteraction("_setup_db", self._setup_db))
 
@@ -76,14 +75,15 @@ class MultiWriterIdGeneratorBase(HomeserverTestCase):
     def _create_id_generator(
         self,
         instance_name: str = "master",
-        writers: Optional[List[str]] = None,
+        writers: list[str] | None = None,
     ) -> MultiWriterIdGenerator:
         def _create(conn: LoggingDatabaseConnection) -> MultiWriterIdGenerator:
             return MultiWriterIdGenerator(
-                conn,
-                self.db_pool,
+                db_conn=conn,
+                db=self.db_pool,
                 notifier=self.hs.get_replication_notifier(),
                 stream_name="test_stream",
+                server_name=self.hs.hostname,
                 instance_name=instance_name,
                 tables=[(table, "instance_name", "stream_id") for table in self.tables],
                 sequence_name="foobar_seq",
@@ -112,7 +112,7 @@ class MultiWriterIdGeneratorBase(HomeserverTestCase):
             self._replicate(instance_name)
 
     def _insert_row(
-        self, instance_name: str, stream_id: int, table: Optional[str] = None
+        self, instance_name: str, stream_id: int, table: str | None = None
     ) -> None:
         """Insert one row as the given instance with given stream_id."""
 
@@ -143,7 +143,7 @@ class MultiWriterIdGeneratorBase(HomeserverTestCase):
         self,
         instance_name: str,
         number: int,
-        table: Optional[str] = None,
+        table: str | None = None,
         update_stream_table: bool = True,
     ) -> None:
         """Insert N rows as the given instance, inserting with stream IDs pulled
