@@ -19,7 +19,7 @@
 #
 #
 
-from typing import Collection, Optional
+from typing import Collection
 
 from twisted.internet.testing import MemoryReactor
 
@@ -101,8 +101,8 @@ class ReceiptTestCase(HomeserverTestCase):
         )
 
     def get_last_unthreaded_receipt(
-        self, receipt_types: Collection[str], room_id: Optional[str] = None
-    ) -> Optional[str]:
+        self, receipt_types: Collection[str], room_id: str | None = None
+    ) -> str | None:
         """
         Fetch the event ID for the latest unthreaded receipt in the test room for the test user.
 
