@@ -20,9 +20,9 @@
 #
 #
 import logging
-from typing import List, Optional, Tuple, cast
+from typing import cast
 
-from twisted.test.proto_helpers import MemoryReactor
+from twisted.internet.testing import MemoryReactor
 
 from synapse.api.constants import EventContentFields, EventTypes, JoinRules, Membership
 from synapse.api.room_versions import RoomVersions
@@ -33,7 +33,7 @@ from synapse.server import HomeServer
 from synapse.storage.databases.main.roommember import extract_heroes_from_room_summary
 from synapse.storage.roommember import MemberSummary
 from synapse.types import UserID, create_requester
-from synapse.util import Clock
+from synapse.util.clock import Clock
 
 from tests import unittest
 from tests.server import TestHomeServer
@@ -133,7 +133,7 @@ class RoomMemberStoreTestCase(unittest.HomeserverTestCase):
         room = self.helper.create_room_as(self.u_alice, tok=self.t_alice)
 
         res = cast(
-            List[Tuple[Optional[str], str]],
+            list[tuple[str | None, str]],
             self.get_success(
                 self.store.db_pool.simple_select_list(
                     "room_memberships",
@@ -165,7 +165,7 @@ class RoomMemberStoreTestCase(unittest.HomeserverTestCase):
         )
 
         res2 = cast(
-            List[Tuple[Optional[str], str]],
+            list[tuple[str | None, str]],
             self.get_success(
                 self.store.db_pool.simple_select_list(
                     "room_memberships",
@@ -408,9 +408,9 @@ class RoomSummaryTestCase(unittest.HomeserverTestCase):
     def _assert_member_summary(
         self,
         actual_member_summary: MemberSummary,
-        expected_member_list: List[str],
+        expected_member_list: list[str],
         *,
-        expected_member_count: Optional[int] = None,
+        expected_member_count: int | None = None,
     ) -> None:
         """
         Assert that the `MemberSummary` object has the expected members.
