@@ -19,11 +19,11 @@
 #
 #
 
-from twisted.test.proto_helpers import MemoryReactor
+from twisted.internet.testing import MemoryReactor
 
 from synapse.server import HomeServer
 from synapse.storage.types import Cursor
-from synapse.util import Clock
+from synapse.util.clock import Clock
 
 from tests import unittest
 
