@@ -20,8 +20,6 @@
 
 """Tests REST events for /rooms paths."""
 
-from typing import List, Optional
-
 from twisted.internet.testing import MemoryReactor
 
 from synapse.api.constants import APP_SERVICE_REGISTRATION_TYPE, LoginType
@@ -249,7 +247,7 @@ class TestMauLimit(unittest.HomeserverTestCase):
         }
     )
     def test_as_trial_days(self) -> None:
-        user_tokens: List[str] = []
+        user_tokens: list[str] = []
 
         def advance_time_and_sync() -> None:
             self.reactor.advance(24 * 60 * 61)
@@ -313,7 +311,7 @@ class TestMauLimit(unittest.HomeserverTestCase):
         )
 
     def create_user(
-        self, localpart: str, token: Optional[str] = None, appservice: bool = False
+        self, localpart: str, token: str | None = None, appservice: bool = False
     ) -> str:
         request_data = {
             "username": localpart,
