@@ -46,7 +46,9 @@ class ResponseCacheTestCase(TestCase):
         self.reactor, self.clock = get_clock()
 
     def with_cache(self, name: str, ms: int = 0) -> ResponseCache:
-        return ResponseCache(self.clock, name, timeout_ms=ms)
+        return ResponseCache(
+            clock=self.clock, name=name, server_name="test_server", timeout_ms=ms
+        )
 
     @staticmethod
     async def instant_return(o: str) -> str:
