@@ -20,7 +20,7 @@
 
 import threading
 from io import BytesIO
-from typing import BinaryIO, Generator, Optional, cast
+from typing import BinaryIO, Generator, cast
 from unittest.mock import NonCallableMock
 
 from zope.interface import implementer
@@ -127,7 +127,7 @@ class FileConsumerTests(unittest.TestCase):
 @implementer(IPullProducer)
 class DummyPullProducer:
     def __init__(self) -> None:
-        self.consumer: Optional[BackgroundFileConsumer] = None
+        self.consumer: BackgroundFileConsumer | None = None
         self.deferred: "defer.Deferred[object]" = defer.Deferred()
 
     def resumeProducing(self) -> None:
@@ -159,7 +159,7 @@ class BlockingBytesWrite:
         self.closed = False
         self.write_lock = threading.Lock()
 
-        self._notify_write_deferred: Optional[defer.Deferred] = None
+        self._notify_write_deferred: defer.Deferred | None = None
         self._number_of_writes = 0
 
     def write(self, write_bytes: bytes) -> None:
